#pragma once
//============================================================================
// Name        : canDataType.h
// Author      :
// Version     :
// Copyright   : SIASUN
// Description : can2.0协议数据结构
//============================================================================

#ifndef CAN_DATA_TYPE_H_
#define CAN_DATA_TYPE_H_

namespace robot {
namespace canbusSpace {

/******************************ICU-RCU控制命令******************************/
/*************控制器上电命令*************/
typedef struct
{
    unsigned int : 32;
    unsigned int uiYSftArmCtrPowReq : 2;	//换挡臂Y电机控制器上电
    unsigned int : 6;
    unsigned int uiStrCtrPowReq : 2;		//转向电机控制器上电
    unsigned int : 2;
    unsigned int uiXSftArmCtrPowReq : 2;	//换挡臂X电机控制器上电
    unsigned int : 2;
    unsigned int uiBrkCtrPowReq : 2;		//制动电机控制器上电
    unsigned int : 2;
    unsigned int uiClchCtrPowReq : 2;		//离合电机控制器上电
    unsigned int : 2;
    unsigned int uiRCUPowReq : 2;			//RCU 上电
    unsigned int : 2;
    unsigned int uiAccCtrPowReq : 2;		//油门电机控制器上电
    unsigned int : 2;
}T_CONTROL_CMD_5F1;

//控制器上电命令
typedef struct
{
    unsigned int uiRCUPowReq;			//RCU上电
    unsigned int uiAccCtrPowReq;		//油门电机控制器上电
    unsigned int uiBrkCtrPowReq;		//制动电机控制器上电
    unsigned int uiClchCtrPowReq;		//离合电机控制器上电
    unsigned int uiStrCtrPowReq;		//转向电机控制器上电
    unsigned int uiXSftArmCtrPowReq;	//换挡臂X电机控制器上电
    unsigned int uiYSftArmCtrPowReq;	//换挡臂Y电机控制器上电
}T_POWER_ON_REQUEST;

#define	POW_REQ_OFF	0		//下电
#define POW_REQ_ON	1		//上电

/*************执行机构使能请求*************/
typedef struct
{
    //unsigned int uiTest: 1;
    //unsigned int : 31;
    unsigned int : 32;
    unsigned int uiCtrMdReq : 3;		//实际系统控制	模式请求
    unsigned int : 5;
    unsigned int uiEstpEnReq : 1;		//机器人Estop使能
    unsigned int : 1;
    unsigned int uiPmpEnReq : 1;		//制动急停气泵使能
    unsigned int : 1;
    unsigned int uiPauseEnReq : 1;	//系统软急停使	能
    unsigned int : 3;
    unsigned int uiYSftMotEnReq : 1;	//换挡臂Y电机使能
    unsigned int : 1;
    unsigned int uiClchMotEnReq : 1;	//离合电机使能
    unsigned int : 1;
    unsigned int uiAccClchCtrReq : 2;	//油门离合器控	制
    unsigned int : 2;
    unsigned int uiAccMotEnReq : 1;	//油门电机使能
    unsigned int : 1;
    unsigned int uiBrkMotEnReq : 1;	//制动电机使能
    unsigned int : 1;
    unsigned int uiStrMotEnReq : 1;	//转向电机使能
    unsigned int : 1;
    unsigned int uiXSftMotEnReq : 1;	//换挡臂X电机使能
    unsigned int : 1;
}T_CONTROL_CMD_5F2;

//执行机构使能请求
typedef struct
{
    unsigned int uiAccMotEnReq;		//油门电机使能
    unsigned int uiBrkMotEnReq;		//制动电机使能
    unsigned int uiStrMotEnReq;		//转向电机使能
    unsigned int uiXSftMotEnReq;	//换挡臂X电机使能
    unsigned int uiYSftMotEnReq;	//换挡臂Y电机使能
    unsigned int uiClchMotEnReq;	//离合电机使能
    unsigned int uiAccClchCtrReq;	//油门离合器控制
    unsigned int uiEstpEnReq;		//机器人Estop使能
    unsigned int uiPmpEnReq;		//制动急停气泵使能
    unsigned int uiPauseEnReq;		//系统软急停使能
    unsigned int uiCtrMdReq;		//实际系统控制模式请求
}T_ACTUATOR_ENABLE_REQUEST;

#define	MOT_EN_REQ_DISABLE		0		//电机禁能
#define	MOT_EN_REQ_ENABLE		1		//电机使能

#define	ACC_CLCH_CTR_REQ_JOINT		0	//油门离合控制器闭合
#define	ACC_CLCH_CTR_REQ_RELEASE	1	//油门离合控制器释放

#define	ESTP_EN_REQ_DISABLE	0 			//机器人Estop禁能
#define	ESTP_EN_REQ_ENABLE 	1 			//机器人Estop使能

#define	PMP_EN_REQ_DISABLE	0			//制动急停气泵禁能
#define	PMP_EN_REQ_ENABLE	1			//制动急停气泵使能

#define	PAUSE_EN_REQ_DISABLE	0		//系统软急停禁能
#define	PAUSE_EN_REQ_ENABLE		1		//系统软急停使能

#define	CTR_MD_REQ_AUTO_LEARN		0	//控制模式请求-自动学习模式
#define	CTR_MD_REQ_HAND_LEARN		1	//控制模式请求-手动学习模式
#define	CTR_MD_REQ_AUTO_DRIVE		2	//控制模式请求-自动驾驶模式
#define	CTR_MD_REQ_HAND_DRIVE		3	//控制模式请求-人工驾驶模式
#define	CTR_MD_REQ_ROBOT_CHECK		4	//控制模式请求-机器人自检模式
#define	CTR_MD_REQ_ROBOT_DEBUG		5	//控制模式请求-机器人调试模式


/*************学习模式请求确认******技术规范和设计汇总不统一******/
typedef struct
{
    unsigned int : 16;
    unsigned int uiSftAutLrnFnsConf : 1;	//档位自动学习完成确认
    unsigned int : 7;
    unsigned int uiClrSftTbl : 1;			//清空挡位学习表
    unsigned int : 1;
    unsigned int uiStpAutLrn : 1;			//终止自动学习
    unsigned int : 5;
    unsigned int uiClrAccTbl : 1;			//清空油门学习表
    unsigned int : 1;
    unsigned int uiClrBrkTbl : 1;			//清空制动学习表
    unsigned int : 1;
    unsigned int uiClrClchTbl : 1;		//清空离合学习表
    unsigned int : 1;
    unsigned int uiClrStrTbl : 1;			//清空转向学习表
    unsigned int : 1;
    unsigned int uiSftPsngLrnReq : 5;		//挡位位置学习请求确认
    unsigned int : 3;
    unsigned int uiPsngLrnReq : 3;		//位置学习请求确认
    unsigned int : 5;
    unsigned int uiSysLrnReq : 3;			//系统学习请求
    unsigned int : 5;
}T_CONTROL_CMD_5F3;

//学习模式请求确认
typedef struct
{
    unsigned int uiSysLrnReq;		//系统学习请求
    unsigned int uiPsngLrnReq;		//位置学习请求确认
    unsigned int uiSftPsngLrnReq;	//挡位位置学习请求确认
    unsigned int uiClrAccTbl;		//清空油门学习表
    unsigned int uiClrBrkTbl;		//清空制动学习表
    unsigned int uiClrClchTbl;		//清空离合学习表
    unsigned int uiClrStrTbl;		//清空转向学习表
    unsigned int uiClrSftTbl;		//清空挡位学习表
    unsigned int uiStpAutLrn;		//终止自动学习
    unsigned int uiSftAutLrnFnsConf;//档位自动学习完成确认
}T_LEARN_MODE_REQUEST;

#define	SYS_LRN_REQ_ACC			1	//油门学习
#define	SYS_LRN_REQ_BRK			2	//制动踏板学习
#define	SYS_LRN_REQ_CLCH		3	//离合踏板学习
#define	SYS_LRN_REQ_STR			4	//转向学习
#define	SYS_LRN_REQ_AUTO_SFT	5	//自动档学习
#define	SYS_LRN_REQ_HAND_SFT	6	//手动档学习

#define	PSNG_LRN_REQ_ZERO		1	//位置学习-零位学习
#define	PSNG_LRN_REQ_MAX		2	//位置学习-最大值学习
#define	PSNG_LRN_REQ_L_MAX		3	//位置学习-左极限学习
#define	PSNG_LRN_REQ_R_MAX		4	//位置学习-右极限学习

#define	SFT_PSNG_LRN_REQ_BETWEEN		0	//挡位位置学习请求-???????????
#define	SFT_PSNG_LRN_REQ_PARK			1	//挡位位置学习请求-停车档（自动档）
#define	SFT_PSNG_LRN_REQ_REVERSE		2	//挡位位置学习请求-倒档
#define	SFT_PSNG_LRN_REQ_NEUTRAL		3	//挡位位置学习请求-空档
#define	SFT_PSNG_LRN_REQ_FORWARD_1		4	//挡位位置学习请求-前进档1(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_2		5	//挡位位置学习请求-前进档2(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_3		6	//挡位位置学习请求-前进档3(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_4		7	//挡位位置学习请求-前进档4(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_5		8	//挡位位置学习请求-前进档5(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_6		9	//挡位位置学习请求-前进档6(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_7		10	//挡位位置学习请求-前进档7(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_8		11	//挡位位置学习请求-前进档8(手动档)
#define	SFT_PSNG_LRN_REQ_DRIVE			12	//挡位位置学习请求-前进档（自动档）
#define	SFT_PSNG_LRN_REQ_ADD			13	//挡位位置学习请求-加
#define	SFT_PSNG_LRN_REQ_SUB			14	//挡位位置学习请求-减
#define	SFT_PSNG_LRN_REQ_UNKOWN			15	//挡位位置学习请求-未定义
#define	SFT_PSNG_LRN_REQ_SPORT			16	//挡位位置学习请求-运动档（自动档）

#define	TBL_CLEAR			1	//清空

#define	STP_AUT_LRN_STOP	1	//终止

#define SFT_AUT_LRN_FNS_CONF_DOWN	1	//学习完成

/*************踏板执行机构控制命令*************/
typedef struct
{
    unsigned int : 7;
    unsigned int uiBrkFocCtrReq : 9;		//制动踏板力控制请求
    int iBrkSpdCtrReq : 8;				//制动踏板速度控制请求
    unsigned int uiBrkCtrMdReq : 2;		//制动踏板控制	模式请求
    unsigned int : 4;
    unsigned int uiBrkPsngCtrReqL : 2;	//制动踏板位置控制请求
    unsigned int uiBrkPsngCtrReqH : 8;	//制动踏板位置控制请求
    int iAccSpdCtrReq : 8;				//油门踏板速度控制请求
    unsigned int : 6;
    unsigned int uiAccPsngCtrReq : 10;	//油门踏板位置控制请求
}T_CONTROL_CMD_5F4;

typedef struct
{
    unsigned int uiBrkPsngCtrReqL : 2;	//制动踏板位置控制请求
    unsigned int uiBrkPsngCtrReqH : 8;	//制动踏板位置控制请求
    unsigned int : 22;
}T_CONTROL_CMD_5F4_SRC;

typedef struct
{
    unsigned int uiBrkPsngCtrReq : 10;	//制动踏板位置控制请求
    unsigned int : 22;
}T_CONTROL_CMD_5F4_DEST;

typedef union
{
    T_CONTROL_CMD_5F4_SRC tCtrCmd5F4Src;
    T_CONTROL_CMD_5F4_DEST tCtrCmd5F4Dest;
}T_CONTROL_CMD_5F4_UNION;

////踏板执行机构控制命令
typedef struct
{
    unsigned int uiAccPsngCtrReq;	//油门踏板位置控制请求
    int iAccSpdCtrReq;				//油门踏板速度控制请求
    unsigned int uiBrkPsngCtrReq;	//制动踏板位置控制请求
    unsigned int uiBrkCtrMdReq;		//制动踏板控制	模式请求
    int iBrkSpdCtrReq;				//制动踏板速度控制请求
    unsigned int uiBrkFocCtrReq;	//制动踏板力控制请求
}T_PEDAL_ACT_CTR_REQ;

#define	BRK_CTR_MD_REQ_SPD	1		//制动踏板控制	模式-速度
#define	BRK_CTR_MD_REQ_FOC	2		//制动踏板控制	模式-力


/*************转向执行机构控制命令*************/
typedef struct
{
    unsigned int : 2;
    int iStrTrqCtrReq : 14;			//方向盘转矩控制请求
    unsigned int : 5;
    int iStrSpdCtrReqL : 11;			//方向盘转速控制请求
    int iStrSpdCtrReqH : 8;			//方向盘转速控制请求
    unsigned int uiStrCtrMdReq : 2;	//方向盘控制模式请求
    unsigned int : 3;
    int iStrAngCtrReq : 19;			//方向盘转角控制请求
}T_CONTROL_CMD_5F5;

typedef struct
{
    int iStrSpdCtrReqL : 11;			//方向盘转速控制请求
    int iStrSpdCtrReqH : 8;			//方向盘转速控制请求
    unsigned int : 13;
}T_CONTROL_CMD_5F5_SRC;

typedef struct
{
    int iStrSpdCtrReq : 19;			//方向盘转速控制请求
    unsigned int : 13;
}T_CONTROL_CMD_5F5_DEST;

typedef union
{
    T_CONTROL_CMD_5F5_SRC tCtrCmd5F5Src;
    T_CONTROL_CMD_5F5_DEST tCtrCmd5F5Dest;
}T_CONTROL_CMD_5F5_UNION;

//转向执行机构控制命令
typedef struct
{
    int iStrAngCtrReq;					//方向盘转角控制请求
    unsigned int uiStrCtrMdReq;			//方向盘控制模式请求
    int iStrSpdCtrReq;					//方向盘转速控制请求
    int iStrTrqCtrReq;					//方向盘转矩控制请求
}T_STR_ACT_CTR_REQ;

#define	STR_CTR_MD_REQ_SPD	1			//方向盘控制模式-转速
#define	STR_CTR_MD_REQ_TRQ	2			//方向盘控制模式-转矩


/*************档位执行机构控制命令*************/
typedef struct
{
    unsigned int : 24;
    int iClchSpdCtrReq : 8;				//离合踏板速度控制请求
    unsigned int : 6;
    unsigned int uiClchPsngCtrReq : 10;	//离合踏板位置控制请求
    unsigned int uiSftMdReq : 2;			//换挡模式请求
    unsigned int : 6;
    unsigned int uiSftPsngCtrReq : 5;		//挡位控制请求
    unsigned int : 3;
}T_CONTROL_CMD_5F6;

//档位执行机构控制命令
typedef struct
{
    unsigned int uiSftPsngCtrReq;		//挡位控制请求
    unsigned int uiSftMdReq;			//换挡模式请求
    unsigned int uiClchPsngCtrReq;		//离合踏板位置控制请求
    int iClchSpdCtrReq;					//离合踏板速度控制请求
}T_SFT_ACT_CTR_REQ;

#define	SFT_PSNG_CTR_REQ_BETWEEN		0	//挡位控制请求-???????????
#define	SFT_PSNG_CTR_REQ_PARK			1	//挡位控制请求-停车档（自动档）
#define	SFT_PSNG_CTR_REQ_REVERSE		2	//挡位控制请求-倒档
#define	SFT_PSNG_CTR_REQ_NEUTRAL		3	//挡位控制请求-空档
#define	SFT_PSNG_CTR_REQ_FORWARD_1		4	//挡位控制请求-前进档1(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_2		5	//挡位控制请求-前进档2(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_3		6	//挡位控制请求-前进档3(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_4		7	//挡位控制请求-前进档4(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_5		8	//挡位控制请求-前进档5(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_6		9	//挡位控制请求-前进档6(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_7		10	//挡位控制请求-前进档7(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_8		11	//挡位控制请求-前进档8(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_9		12	//挡位控制请求-前进档9(手动档)
#define	SFT_PSNG_CTR_REQ_ADD			13	//挡位控制请求-加
#define	SFT_PSNG_CTR_REQ_SUB			14	//挡位控制请求-减
#define	SFT_PSNG_CTR_REQ_MANUAL			15	//挡位控制请求-???????????手动选择
#define	SFT_PSNG_CTR_REQ_SPORT			16	//挡位控制请求-运动档（自动档）

#define SFT_PSNG_CTR_REQ_DRIVE			4

#define	SFT_MD_REQ_3_SECOND		0			//换挡模式-平滑型
#define	SFT_MD_REQ_2_SECOND		1			//换挡模式-快速型
#define	SFT_MD_REQ_1_SECOND		2			//换挡模式-激进型


/************* 机器人自检控制命令******技术规范和设计汇总不统一*******/
typedef struct
{
    unsigned int : 32;
    unsigned int : 24;
    unsigned int uiRbtSlfChkReq : 2;	//机器人自检请求
    unsigned int : 6;
}T_CONTROL_CMD_5F7;

//机器人自检控制命令
typedef struct
{
    unsigned int uiRbtSlfChkReq;	//机器人自检请求
}T_RBT_SLF_CHK_CTR_REQ;

#define	RBT_SLF_CHK_REQ_START	1	//机器人自检-开始
#define	RBT_SLF_CHK_REQ_END	2	//机器人自检-终止


/************* 机器人手动控制命令******设计汇总未定义*******/
typedef struct
{
    unsigned int : 32;
    unsigned int : 16;
    unsigned int uiMotMove : 2;		//电机转向
    unsigned int : 6;
    unsigned int uiAccMotCtrl : 1;	//油门电机控制请求
    unsigned int uiBrkMotCtrl : 1;	//制动电机控制请求
    unsigned int uiXSftMotCtrl : 1;	//X轴控制请求
    unsigned int uiYSftMotCtrl : 1;	//Y轴控制请求
    unsigned int uiStrMotCtrl : 1;	//转向控制请求
    unsigned int uiClchMotCtrl : 1;	//离合控制请求
    unsigned int : 2;
}T_CONTROL_CMD_5F8;

//机器人手动控制命令
typedef struct
{
    unsigned int uiAccMotCtrl;		//油门电机控制请求
    unsigned int uiBrkMotCtrl;		//制动电机控制请求
    unsigned int uiXSftMotCtrl;		//X轴控制请求
    unsigned int uiYSftMotCtrl;		//Y轴控制请求
    unsigned int uiStrMotCtrl;		//转向控制请求
    unsigned int uiClchMotCtrl;		//离合控制请求
    unsigned int uiMotMove;			//电机转向
}T_RBT_DRIVER_CTR;

#define	MOT_CTR_DISABLE		0	//电机控制请求-禁能
#define	MOT_CTR_ENABLE		1	//电机控制请求-使能

#define	MOT_MOVE_FORWARD	1	//电机正转
#define	MOT_MOVE_REVERSE	2	//电机反转


/************* 设备管理******设计汇总未定义*******/
typedef struct
{
    unsigned int : 8;
    unsigned int uiFltClr : 1;		//故障清除
    unsigned int : 7;
    unsigned int uiYSftMotInt : 1;	//档位Y电机初始化
    unsigned int : 7;
    unsigned int uiXSftMotInt : 1;	//档位X电机初始化
    unsigned int : 7;
    unsigned int uiStrMotInt : 1;		//转向电机初始化
    unsigned int : 7;
    unsigned int uiClchMotInt : 1;	//离合电机初始化
    unsigned int : 7;
    unsigned int uiBrkMotInt : 1;		//制动电机初始化
    unsigned int : 7;
    unsigned int uiAccMotInt : 1;		//油门电机初始化
    unsigned int : 7;
}T_CONTROL_CMD_5F9;

//设备管理
typedef struct
{
    unsigned int uiAccMotInt;		//油门电机初始化
    unsigned int uiBrkMotInt;		//制动电机初始化
    unsigned int uiClchMotInt;		//离合电机初始化
    unsigned int uiStrMotInt;		//转向电机初始化
    unsigned int uiXSftMotInt;		//档位X电机初始化
    unsigned int uiYSftMotInt;		//档位Y电机初始化
    unsigned int uiFltClr;			//故障清除
}T_RBT_MANAGER;

#define	MOT_INT_ZERO		1		//电机初始化-标零
#define	FLT_CLR_RESET		0		//故障消除-无效


/*************上位机监控******设计汇总未定义*******/
typedef struct
{
    unsigned int : 32;
    unsigned int : 16;
    unsigned int uiICUClsReq : 1;		//上位机关闭请求
    unsigned int : 7;
    unsigned int uiICUAlive : 4;		//上位机心跳
    unsigned int : 4;
}T_CONTROL_CMD_5FA;

//上位机监控
typedef struct
{
    unsigned int uiICUAlive;		//上位机心跳
    unsigned int uiICUClsReq;		//上位机关闭请求
}T_ICU_MONITOR;

#define	ICU_CLS_REQ_CLOSE	1		//正常关闭请求


/*************执行机构加速度控制请求******设计汇总未定义*******/
typedef struct
{
    unsigned int : 20;
    unsigned int uiStrAccCtrReq : 12;		//转向机构加速度控制请求
    unsigned int : 4;
    unsigned int uiBrkAccCtrReq : 12;		//制动机构加速度控制请求
    unsigned int : 4;
    unsigned int uiAccAccCtrReq : 12;		//油门机构加速度控制请求
}T_CONTROL_CMD_5FB;

//执行机构加速度控制请求
typedef struct
{
    unsigned int uiAccAccCtrReq;		//油门机构加速度控制请求
    unsigned int uiBrkAccCtrReq;		//制动机构加速度控制请求
    unsigned int uiStrAccCtrReq;		//转向机构加速度控制请求
}T_ACT_ACC_CTR_REQ;


/*************执行机构限值设置******设计汇总未定义*******/
typedef struct
{
    unsigned int uiSftType : 2;  //挡位类型
    unsigned int : 11;
    int iStrMaxLimt : 19;					//转向极限值
    unsigned int : 6;
    unsigned int uiBrkMaxLimt : 10;		//制动行程最大限值
    unsigned int : 6;
    unsigned int uiAccMaxLimt : 10;		//油门行程最大限值
}T_CONTROL_CMD_5FC;

//执行机构限值设置
typedef struct
{
    unsigned int uiSftType;   //挡位类型
    unsigned int uiAccMaxLimt;		//油门行程最大限值
    unsigned int uiBrkMaxLimt;		//制动行程最大限值
    int iStrMaxLimt;				//转向极限值
}T_ACT_CTR_LIMT;
/*************力矩保护设置*******/
typedef struct
{
    unsigned int : 32;
    unsigned int : 15;
    unsigned int uiStrTorqSwitch : 1;					//转向力矩限制开关
    unsigned int : 7;
    unsigned int uiStrTorqLimit : 9;		//转向力矩限值
}T_CONTROL_CMD_5FD;

//力矩保护设置
typedef struct
{
    unsigned int uiStrTorqSwitch;					//转向力矩限制开关
    unsigned int uiStrTorqLimit;		//转向力矩限值
}T_TRQ_PRTCT_STP;

/*************制动机构力时间控制设置*******/
typedef struct
{
    unsigned int : 21;
    unsigned int uiBrkTm : 11;   //控制时间
    unsigned int : 6;
    unsigned int uiBscFrc : 10;  //力的基准值
    unsigned int : 2;
    unsigned int uiFrcSpd : 14;		//力增长速率
}T_CONTROL_CMD_5FE;

//制动机构力时间控制设置
typedef struct
{
    unsigned int uiBrkTm;					//控制时间
    unsigned int uiBscFrc;     //力的基准值
    unsigned int uiFrcSpd;		//力增长速率
}T_BRK_FRC_CTR_STP;

/*************转向三角波*******/
typedef struct
{
    unsigned int : 5;
    unsigned int uiBrkTm : 11;   //控制时间
    unsigned int : 1;
    int uiStrPstBsc : 11;   //方向盘位置基准值
    unsigned int : 1;
    int uiStrSpdkL : 3;  //方向盘转速
    int uiStrSpdkH : 8;  //方向盘转速
    unsigned int uiStrNum : 4;  //个数
    unsigned int : 3;
    unsigned int uiStrTT : 9;		//脉宽
    int uiStrTA : 8;		//峰值
}T_CONTROL_CMD_7F3;

typedef struct
{
    int uiStrSpdkL : 3;  //方向盘转速
    int uiStrSpdkH : 8;  //方向盘转速
    int : 21;
}T_CONTROL_CMD_7F3_SRC;

typedef struct
{
    int uiStrSpdk : 11;			//方向盘转速
    int : 21;
}T_CONTROL_CMD_7F3_DEST;

typedef union
{
    T_CONTROL_CMD_7F3_SRC tCtrCmd7F3Src;
    T_CONTROL_CMD_7F3_DEST tCtrCmd7F3Dest;
}T_CONTROL_CMD_7F3_UNION;

//转向三角波
typedef struct
{
    unsigned int uiBrkTm;   //控制时间
    int uiStrPstBsc;   //方向盘位置基准值
    int uiStrSpdk;  //方向盘转速
    unsigned int uiStrNum;  //个数
    unsigned int uiStrTT;		//脉宽
    int uiStrTA;		//峰值
}T_STR_TRNGL_LN_CTR;

/*************转向正弦控制*******/
typedef struct
{
    unsigned int uiBrkTm : 10;   //控制时间
    unsigned int : 1;
    unsigned int uiStrDt : 10;   //迟滞时间
    unsigned int : 1;
    unsigned int uiStrA : 9;  //幅值
    unsigned int : 1;
    int uiStrFi : 9;  //相位差
    unsigned int : 2;
    unsigned int uiStrFrtb : 10;		//频率基值
    unsigned int : 1;
    unsigned int uiStrFrtk : 10;		//频率斜率
}T_CONTROL_CMD_7F4;

//转向正弦控制
typedef struct
{
    unsigned int uiBrkTm;   //控制时间
    unsigned int uiStrDt;   //迟滞时间
    unsigned int uiStrA;  //幅值
    int uiStrFi;  //相位差
    unsigned int uiStrFrtb;		//频率基值
    unsigned int uiStrFrtk;		//频率斜率
}T_STR_SIN_CTR;

/*************油门机构位置时间控制*******/
typedef struct
{
    unsigned int : 21;
    unsigned int uiBrkTm : 11;   //控制时间
    unsigned int : 5;
    unsigned int uiAccPstBsc : 11;   //油门位置基准值
    unsigned int : 6;
    unsigned int uiAccSpdP : 10;  //油门速度
}T_CONTROL_CMD_7F5;

//油门机构位置时间控制
typedef struct
{
    unsigned int uiBrkTm : 11;   //控制时间
    unsigned int uiAccPstBsc : 11;   //油门位置基准值
    unsigned int uiAccSpdP : 10;  //油门速度
}T_ACC_PSTN_CTR_STP;

/*************方向盘位置时间序列控制*******/
typedef struct
{
    unsigned int : 5;
    int uiStrAngReq : 19;   //方向盘转角
    unsigned int uiRelTimeL : 8;   //相对时间
    unsigned int uiRelTimeH : 8;   //相对时间
    unsigned int uiNum : 8;  //实际点序号
    unsigned int uiTotalNum : 8;  //总个数
    unsigned int uiSqncID : 8;  //序列ID
}T_CONTROL_CMD_7F6;

typedef struct
{
    unsigned int uiRelTimeL : 8;   //相对时间
    unsigned int uiRelTimeH : 8;   //相对时间
    unsigned int : 16;
}T_CONTROL_CMD_7F6_SRC;

typedef struct
{
    unsigned int uiRelTime : 16;				//相对时间
    unsigned int : 16;
}T_CONTROL_CMD_7F6_DEST;

typedef union
{
    T_CONTROL_CMD_7F6_SRC tCtrCmd7F6Src;
    T_CONTROL_CMD_7F6_DEST tCtrCmd7F6Dest;
}T_CONTROL_CMD_7F6_UNION;

//方向盘位置时间序列控制
typedef struct
{
    int uiStrAngReq;   //方向盘转角
    unsigned int uiRelTime;   //相对时间
    unsigned int uiNum;  //实际点序号
    unsigned int uiTotalNum;  //总个数
    unsigned int uiSqncID;  //序列ID
}T_STR_PSTN_TM_CTR;

/*************车辆一键启动*******/
typedef struct
{
    unsigned int : 32;
    unsigned int : 30;
    unsigned int uiVehSwReq : 2;   //车辆一键启动
}T_CONTROL_CMD_7F7;

//车辆一键启动
typedef struct
{
    unsigned int uiVehSwReq;   //车辆一键启动
}T_VEHICLE_START;


/******************************ICU-RCU状态输入******************************/
/*************车辆状态1*******技术规范添加了部分定义******/
typedef struct
{
    unsigned int uiVehHBrkStat : 1;	//车辆手刹状态
    unsigned int : 2;
    unsigned int uiVehSwStat : 2;		//实际车辆启动状态
    unsigned int : 3;
    unsigned int uiSftPsng : 5;		//实际挡位
    unsigned int : 1;
    unsigned int uiVehSftMd : 2;		//挡位模式
    unsigned int uiDrvSftTrgtGear : 4;//手动挡目标挡位
    unsigned int : 2;
    unsigned int uiClchPsng : 10;		//实际离合器踏板行程
    unsigned int : 6;
    unsigned int uiBrkPsng : 10;		//实际制动踏板行程
    unsigned int : 6;
    unsigned int uiAccPsng : 10;		//实际油门踏板行程
}T_STATE_IN_6F1;

//车辆状态1
typedef struct
{
    unsigned int uiAccPsng;			//实际油门踏板行程
    unsigned int uiBrkPsng;			//实际制动踏板行程
    unsigned int uiClchPsng;		//实际离合器踏板行程
    unsigned int uiDrvSftTrgtGear;	//手动挡目标挡位
    unsigned int uiVehSftMd;		//挡位模式
    unsigned int uiSftPsng;			//实际挡位
    unsigned int uiVehHBrkStat;		//车辆手刹状态
    unsigned int uiVehSwStat;		//实际车辆启动状态
}T_VEHICLE_STATUS_1;

#define	SFT_PSNG_BETWEEN		0	//实际挡位-两档间
#define	SFT_PSNG_PARK			1	//实际挡位-停车档（自动档）
#define	SFT_PSNG_REVERSE		2	//实际挡位-倒档
#define	SFT_PSNG_NEUTRAL		3	//实际挡位-空档
#define SFT_PSNG_DRIVE			4	//实际档位-D档
#define	SFT_PSNG_FORWARD_1		4	//实际挡位-前进档1(手动档)
#define	SFT_PSNG_FORWARD_2		5	//实际挡位-前进档2(手动档)
#define	SFT_PSNG_FORWARD_3		6	//实际挡位-前进档3(手动档)
#define	SFT_PSNG_FORWARD_4		7	//实际挡位-前进档4(手动档)
#define	SFT_PSNG_FORWARD_5		8	//实际挡位-前进档5(手动档)
#define	SFT_PSNG_FORWARD_6		9	//实际挡位-前进档6(手动档)
#define	SFT_PSNG_FORWARD_7		10	//实际挡位-前进档7(手动档)
#define	SFT_PSNG_FORWARD_8		11	//实际挡位-前进档8(手动档)
#define	SFT_PSNG_UNKOWN			15	//实际挡位-???????????


#define	SFT_PSNG_ADD			12	//实际挡位-???????????
#define	SFT_PSNG_SUB			13	//实际挡位-???????????
#define	SFT_PSNG_MANUAL			14	//实际挡位-???????????

#define VEH_SFT_MD_NO_ACT		0	//挡位模式-???????????
#define VEH_SFT_MD_M			1	//挡位模式-???????????
#define VEH_SFT_MD_ELEC_SLT		2	//?挡位模-式??????????

#define	DRV_SFT_TRG_FORWARD_1	1	//手动挡目标挡位-前进档1(手动档)
#define	DRV_SFT_TRG_FORWARD_2	2	//手动挡目标挡位-前进档2(手动档)
#define	DRV_SFT_TRG_FORWARD_3	3	//手动挡目标挡位-前进档3(手动档)
#define	DRV_SFT_TRG_FORWARD_4	4	//手动挡目标挡位-前进档4(手动档)
#define	DRV_SFT_TRG_FORWARD_5	5	//手动挡目标挡位-前进档5(手动档)
#define	DRV_SFT_TRG_FORWARD_6	6	//手动挡目标挡位-前进档6(手动档)
#define	DRV_SFT_TRG_FORWARD_7	7	//手动挡目标挡位-前进档7(手动档)
#define	DRV_SFT_TRG_FORWARD_8	8	//手动挡目标挡位-前进档8(手动档)

#define	VEH_HBRK_STAT_RELEASE	0	//车辆手刹状态-释放
#define	VEH_HBRK_STAT_HOLD		1	//车辆手刹状态-闭合

#define	VEH_SW_STAT_LOCK	0		//实际车辆启动状态-钥匙插入状态
#define	VEH_SW_STAT_ACC		1		//实际车辆启动状态-接通部分电器电源
#define	VEH_SW_STAT_ON		2		//实际车辆启动状态-接通所有电源
#define	VEH_SW_STAT_START	3		//实际车辆启动状态-点火档


/*************车辆状态2*******技术规范添加了部分定义******/
typedef struct
{
    unsigned int : 3;
    unsigned int uiEngSpd : 13;		//实际发动机转速
    unsigned int uiVehSpd : 8;		//实际车速
    unsigned int : 4;
    int iStrAngGrdL : 4;				//实际方向盘角度梯度
    int iStrAngGrdH : 8;				//实际方向盘角度梯度
    unsigned int : 5;
    int iStrAng : 19;					//实际方向盘转角
}T_STATE_IN_6F2;

typedef struct
{

    int iStrAngGrdL : 4;				//实际方向盘角度梯度
    int iStrAngGrdH : 8;				//实际方向盘角度梯度
    int : 20;						//实际方向盘转角
}T_STATE_IN_6F2_SRC;

typedef struct
{
    int iStrAngGrd : 12;				//实际方向盘角度梯度
    int : 20;
}T_STATE_IN_6F2_DEST;

typedef union
{
    T_STATE_IN_6F2_SRC tStateIn6F2Src;
    T_STATE_IN_6F2_DEST tStateIn6F2Dest;
}T_STATE_IN_6F2_UNION;

//车辆状态2
typedef struct
{
    int iStrAng;					//实际方向盘转角
    int iStrAngGrd;					//实际方向盘角度梯度
    unsigned int uiVehSpd;			//实际车速
    unsigned int uiEngSpd;			//实际发动机转速
}T_VEHICLE_STATUS_2;


/*************车辆状态3*******技术规范添加了部分定义******/
typedef struct
{
    unsigned int : 24;
    unsigned int uiVehBatVlt : 8;		//实际车辆蓄电池电压
    unsigned int : 2;
    unsigned int uiVehBatCur : 22;	//实际车辆蓄电池电流
    unsigned int uiVehBatQut : 8;		//实际车辆蓄电池电量
}T_STATE_IN_6F3;

//车辆状态3
typedef struct
{
    unsigned int uiVehBatQut;		//实际车辆蓄电池电量
    unsigned int uiVehBatCur;		//实际车辆蓄电池电流
    unsigned int uiVehBatVlt;		//实际车辆蓄电池电压
}T_VEHICLE_STATUS_3;


/*************工控机状态*******技术规范修改了定义******/
typedef struct
{
    unsigned int : 32;
    unsigned int : 24;
    unsigned int uiCmpStat : 2;		//实际工控机工作状态
    unsigned int : 6;
}T_STATE_IN_6F4;

//工控机状态
typedef struct
{
    unsigned int uiCmpStat;			//实际工控机工作状态
}T_COMPUTER_STATUS;

#define	CMP_STAT_ON			0		//上电
#define	CMP_STAT_RUN		1		//运行中
#define	CMP_STAT_ERR		2		//异常


/******************************ICU-RCU状态输出******************************/
/*************机器人执行机构实际位置*******技术规范修改了定义******/
typedef struct
{
    unsigned int : 21;
    int iCcltStrActAngL : 11;				//方向盘机构转角
    int iCcltStrActAngH : 8;				//方向盘机构转角
    unsigned int uiCcltClchPsng : 7;		//离合踏板机构位置
    unsigned int : 1;
    unsigned int uiCcltBrkActPsng : 8;	//制动踏板机构位置
    unsigned int uiCcltAccActPsng : 7;	//油门踏板机构位置
    unsigned int : 1;
}T_STATE_OUT_6E0;


typedef struct
{
    int iCcltStrActAngL : 11;				//方向盘机构转角
    int iCcltStrActAngH : 8;				//方向盘机构转角
    unsigned int : 13;
}T_STATE_OUT_6E0_SRC;

typedef struct
{
    int iCcltStrActAng : 19;				//方向盘机构转角
    unsigned int : 13;
}T_STATE_OUT_6E0_DEST;

typedef union
{
    T_STATE_OUT_6E0_SRC tStateOut6E0Src;
    T_STATE_OUT_6E0_DEST tStateOut6E0Dest;
}T_STATE_OUT_6E0_UNION;
//机器人执行机构实际位置
typedef struct
{
    unsigned int uiCcltAccActPsng;		//油门踏板机构位置
    unsigned int uiCcltBrkActPsng;		//制动踏板机构位置
    unsigned int uiCcltClchPsng;		//离合踏板机构位置
    int iCcltStrActAng;					//方向盘机构转角
}T_RBT_ACT_PSNG;


/*************(油门和制动)踏板学习位置反馈*******技术规范修改了定义******/
typedef struct
{
    unsigned int : 32;
    unsigned int uiClbtBrkPsngMax : 8;	//标定制动踏板最大行程时机器人的行程
    unsigned int uiClbtBrkPsng0 : 8;		//标定制动踏板0点时的机器人行程
    unsigned int uiClbtAccPsngMax : 7;	//标定油门踏板最大行程时机器人的行程
    unsigned int : 1;
    unsigned int uiClbtAccPsng0 : 7;		//标定油门踏板0点时机器人的行程
    unsigned int : 1;
}T_STATE_OUT_6E1;

//(油门和制动)踏板学习位置反馈
typedef struct
{
    unsigned int uiClbtAccPsng0;		//标定油门踏板0点时机器人的行程
    unsigned int uiClbtAccPsngMax;		//标定油门踏板最大行程时机器人的行程
    unsigned int uiClbtBrkPsng0;		//标定制动踏板0点时的机器人行程
    unsigned int uiClbtBrkPsngMax;		//标定制动踏板最大行程时机器人的行程
}T_ACC_BRK_PSNG;


/*************转向学习位置反馈*************/
typedef struct
{
    unsigned int : 9;
    int iClbtStrAngRtMax : 17;	//标定的方向盘右最大转角时机器人的方向盘转角
    int iClbtStrAngLftMaxL : 6;	//标定的方向盘左最大转角时机器人的方向盘转角
    int iClbtStrAngLftMaxH : 13;	//标定的方向盘左最大转角时机器人的方向盘转角
    int iClbtStrAng0 : 19;		//标定的方向盘零点角度时机器人的方向盘转角
}T_STATE_OUT_6E2;

typedef struct
{
    int iClbtStrAngLftMaxL : 6;	//标定的方向盘左最大转角时机器人的方向盘转角
    int iClbtStrAngLftMaxH : 13;	//标定的方向盘左最大转角时机器人的方向盘转角
    unsigned int : 13;
}T_STATE_OUT_6E2_SRC;

typedef struct
{
    int iClbtStrAngLftMax : 19;	//标定的方向盘左最大转角时机器人的方向盘转角
    unsigned int : 13;
}T_STATE_OUT_6E2_DEST;

typedef union
{
    T_STATE_OUT_6E2_SRC tStateOut6E2Src;
    T_STATE_OUT_6E2_DEST tStateOut6E2Dest;
}T_STATE_OUT_6E2_UNION;

//转向学习位置反馈
typedef struct
{
    int iClbtStrAng0;		//标定的方向盘零点角度时机器人的方向盘转角
    int iClbtStrAngLftMax;	//标定的方向盘左最大转角时机器人的方向盘转角
    int iClbtStrAngRtMax;	//标定的方向盘右最大转角时机器人的方向盘转角
}T_STR_LRN_PSNG;


/*************档位学习位置反馈1*******技术规范修改了定义******/
typedef struct
{
    unsigned int : 1;
    int iDXMotPst : 9;		//D挡X电机位置
    int iNYMotPst : 9;		//N挡Y电机位置
    int iNXMotPst : 9;		//N挡X电机位置
    int iRYMotPstL : 4;		//R挡Y电机位置
    int iRYMotPstH : 5;		//R挡Y电机位置
    int iRXMotPst : 9;		//R挡X电机位置
    int iPYMotPst : 9;		//P挡Y电机位置
    int iPXMotPst : 9;		//P挡X电机位置
}T_STATE_OUT_6E3;

typedef struct
{
    int iRYMotPstL : 4;		//R挡Y电机位置
    int iRYMotPstH : 5;		//R挡Y电机位置
    unsigned int : 23;
}T_STATE_OUT_6E3_SRC;

typedef struct
{
    int iRYMotPst : 9;		//R挡Y电机位置
    unsigned int : 23;
}T_STATE_OUT_6E3_DEST;

typedef union
{
    T_STATE_OUT_6E3_SRC tStateOut6E3Src;
    T_STATE_OUT_6E3_DEST tStateOut6E3Dest;
}T_STATE_OUT_6E3_UNION;

//档位学习位置反馈1
typedef struct
{
    int iPXMotPst;		//P挡X电机位置
    int iPYMotPst;		//P挡Y电机位置
    int iRXMotPst;		//R挡X电机位置
    int iRYMotPst;		//R挡Y电机位置
    int iNXMotPst;		//N挡X电机位置
    int iNYMotPst;		//N挡Y电机位置
    int iDXMotPst;		//D挡X电机位置
}T_SFT_LRN_PST_1;


/*************控制器上电状态*************/
typedef struct
{
    unsigned int : 32;
    unsigned int : 16;
    unsigned int uiStrCtrPowStat : 1;		//转向电机控制器上电状态
    unsigned int : 1;
    unsigned int uiXSftArmCtrPowStat : 1;	//换挡臂X电机控制器上电状态
    unsigned int : 1;
    unsigned int uiYSftArmCtrPowStat : 1;	//换挡臂Y电机控制器上电状态
    unsigned int : 3;
    unsigned int uiRCUPowStat : 1;		//RCU上电状态
    unsigned int : 1;
    unsigned int uiAccCtrPowStat : 1;		//油门踏板电机控制器上电状态
    unsigned int : 1;
    unsigned int uiBrkCtrPowStat : 1;		//制动电机控制器上电状态
    unsigned int : 1;
    unsigned int uiClchCtrPowStat : 1;	//离合电机控制器上电状态
    unsigned int : 1;
}T_STATE_OUT_6E4;

//控制器上电状态
typedef struct
{
    unsigned int uiRCUPowStat;			//RCU上电状态
    unsigned int uiAccCtrPowStat;		//油门踏板电机控制器上电状态
    unsigned int uiBrkCtrPowStat;		//制动电机控制器上电状态
    unsigned int uiClchCtrPowStat;		//离合电机控制器上电状态
    unsigned int uiStrCtrPowStat;		//转向电机控制器上电状态
    unsigned int uiXSftArmCtrPowStat;	//换挡臂X电机控制器上电状态
    unsigned int uiYSftArmCtrPowStat;	//换挡臂Y电机控制器上电状态
}T_CTR_POW_STATUS;

#define	POW_STAT_OFF	0	//未上电
#define	POW_STAT_ON		1	//已上电


/*************执行机构使能状态*************/
typedef struct
{
    unsigned int : 32;
    unsigned int : 8;
    unsigned int uiBrkHVFFStat : 1;		//制动机构手阀状态
    unsigned int : 7;
    unsigned int uiYSftMotEnStat : 1;		//换挡臂Y电机使能状态
    unsigned int : 1;
    unsigned int uiClchMotEnStat : 1;		//离合电机使能状态
    unsigned int : 1;
    unsigned int uiAccClchCtrStat : 1;	//油门踏板离合器通电状态
    unsigned int : 1;
    unsigned int uiPauseEnStat : 1;		//系统软急停使能状态
    unsigned int : 1;
    unsigned int uiAccMotEnStat : 1;		//油门踏板电机使能状态
    unsigned int : 1;
    unsigned int uiBrkMotEnStat : 1;		//制动电机使能状态
    unsigned int : 1;
    unsigned int uiStrMotEnStat : 1;		//转向电机使能状态
    unsigned int : 1;
    unsigned int uiXSftMotEnStat : 1;		//换挡臂X电机使能状态
    unsigned int : 1;
}T_STATE_OUT_6E5;

//执行机构使能状态
typedef struct
{
    unsigned int uiAccMotEnStat;		//油门踏板电机使能状态
    unsigned int uiBrkMotEnStat;		//制动电机使能状态
    unsigned int uiStrMotEnStat;		//转向电机使能状态
    unsigned int uiXSftMotEnStat;		//换挡臂X电机使能状态
    unsigned int uiYSftMotEnStat;		//换挡臂Y电机使能状态
    unsigned int uiClchMotEnStat;		//离合电机使能状态
    unsigned int uiAccClchCtrStat;		//油门踏板离合器通电状态
    unsigned int uiPauseEnStat;			//系统软急停使能状态
    unsigned int uiBrkHVFFStat;			//制动机构手阀状态
}T_ACT_ENABLE_STATUS;

#define	MOT_EN_STAT_DISABLE		0		//电机禁能
#define	MOT_EN_STAT_ENABLE		1		//电机使能

#define	ACC_CLCH_CTR_STAT_JOINT		0	//油门离合控制器闭合(通电)
#define	ACC_CLCH_CTR_STAT_RELEASE	1	//油门离合控制器释放(断电)

#define	PAUSE_EN_STAT_DISABLE	0		//系统软急停禁能
#define	PAUSE_EN_STAT_ENABLE	1		//系统软急停使	能

#define	BRK_HVFF_STAT_OPNE		0		//制动机构手阀状态-打开
#define	BRK_HVFF_STAT_CLOSE		1		//制动机构手阀状态-关闭


/*************油门控制状态反馈*******技术规范修改了定义******/
typedef struct
{
    unsigned int : 32;
    int iAccMotTrq : 5;		//计算油门踏板电机转矩
    int iAccDspSpd : 11;		//油门踏板位移速度
    unsigned int : 3;
    int iAccMotSpd : 13;		//实际油门踏板电机转速
}T_STATE_OUT_6E6;

//油门控制状态反馈
typedef struct
{
    int iAccMotSpd;		//实际油门踏板电机转速
    int iAccDspSpd;		//油门踏板位移速度
    int iAccMotTrq;		//计算油门踏板电机转矩
}T_ACC_CTR_STATUS;


/*************制动踏板控制状态反馈*******技术规范修改了定义******/
typedef struct
{
    unsigned int uiBrkCtrMd : 2;	//制动踏板控制模式请求
    unsigned int : 6;
    int iBrkMotTrq : 5;			//计算制动踏板电机转矩
    unsigned int : 10;
    int iBrkFoc : 9;				//计算制动踏板力
    unsigned int : 7;
    int iBrkDspSpd : 9;			//制动踏板位移速度
    unsigned int : 3;
    int iBrkMotSpd : 13;			//实际制动踏板电机转速
}T_STATE_OUT_6E7;

//制动踏板控制状态反馈
typedef struct
{
    int iBrkMotSpd;				//实际制动踏板电机转速
    int iBrkDspSpd;				//制动踏板位移速度
    int iBrkFoc;				//计算制动踏板力
    int iBrkMotTrq;				//计算制动踏板电机转矩
    unsigned int uiBrkCtrMd;	//制动踏板控制模式请求
}T_BRK_CTR_STATUS;

#define	BRK_CTR_MD_SPD	1		//制动踏板控制	模式-速度
#define	BRK_CTR_MD_FOC	2		//制动踏板控制	模式-力


/*************离合踏板控制状态反馈*******技术规范修改了定义******/
typedef struct
{
    unsigned int : 16;
    int iClchMotTrq : 5;			//实际离合踏板电机转矩
    unsigned int : 10;
    unsigned int uiClchFocL : 1;	//计算离合踏板力
    unsigned int uiClchFocH : 8;	//计算离合踏板力
    int iClchDspSpd : 8;			//离合踏板位移速度
    unsigned int : 3;
    int iClchMotSpd : 13;			//实际离合踏板电机转速
}T_STATE_OUT_6E8;


typedef struct
{
    unsigned int uiClchFocL : 1;	//计算离合踏板力
    unsigned int uiClchFocH : 8;	//计算离合踏板力
    unsigned int : 23;
}T_STATE_OUT_6E8_SRC;

typedef struct
{
    unsigned int uiClchFoc : 9;	//计算离合踏板力
    unsigned int : 23;
}T_STATE_OUT_6E8_DEST;

typedef union
{
    T_STATE_OUT_6E8_SRC tStateOut6E8Src;
    T_STATE_OUT_6E8_DEST tStateOut6E8Dest;
}T_STATE_OUT_6E8_UNION;
//离合踏板控制状态反馈
typedef struct
{
    int iClchMotSpd;			//实际离合踏板电机转速
    int iClchDspSpd;			//离合踏板位移速度
    unsigned int uiClchFoc;		//计算离合踏板力
    int iClchMotTrq;			//实际离合踏板电机转矩
}T_CLCH_CTR_STATUS;


/*************转向控制状态反馈*******技术规范修改了定义******/
typedef struct
{
    unsigned int uiStrCtrMd : 2;	//方向盘控制模式
    unsigned int : 10;
    int iCcltStrSpd : 12;			//方向盘转速
    unsigned int : 3;
    int iStrMotSpdL : 5;			//实际转向电机转速
    int iStrMotSpdH : 8;			//实际转向电机转速
    int iStrMotTrq : 5;			//计算转向电机转矩
    unsigned int : 5;
    int iStrTrq : 14;				//方向盘转矩
}T_STATE_OUT_6E9;

typedef struct
{
    int iStrMotSpdL : 5;			//实际转向电机转速
    int iStrMotSpdH : 8;			//实际转向电机转速
    unsigned int : 19;
}T_STATE_OUT_6E9_SRC;

typedef struct
{
    int iStrMotSpd : 13;			//实际转向电机转速
    unsigned int : 19;
}T_STATE_OUT_6E9_DEST;

typedef union
{
    T_STATE_OUT_6E9_SRC tStateOut6E9Src;
    T_STATE_OUT_6E9_DEST tStateOut6E9Dest;
}T_STATE_OUT_6E9_UNION;

//转向控制状态反馈
typedef struct
{
    int iStrTrq;				//方向盘转矩
    int iStrMotTrq;				//计算转向电机转矩
    int iStrMotSpd;				//实际转向电机转速
    int iCcltStrSpd;			//方向盘转速
    unsigned int uiStrCtrMd;	//方向盘控制模式
}T_STR_STATUS;

#define	STR_CTR_MD_SPD	1		//方向盘控制模式-转速
#define	STR_CTR_MD_TRQ	2		//方向盘控制模式-转矩


/*************档位控制状态反馈*******技术规范修改了定义******/
typedef struct
{
    unsigned int uiSftSpdType : 2;	//换挡完成类型
    unsigned int : 1;
    unsigned int uiRbtSftPsng : 5;	//机器人控制的挡位位置
    unsigned int : 2;
    int iYSftMotSpd : 11;				//实际换挡臂Y电机转速
    unsigned int : 1;
    int iXSftMotSpdL : 10;			//实际换挡臂X电机转速
    int iXSftMotSpdH : 1;				//实际换挡臂X电机转速
    unsigned int : 2;
    int iYSftMotTrq : 5;				//计算换挡臂Y电机转矩
    int iXSftMotTrq : 5;				//计算换挡臂X电机转矩
    unsigned int : 1;
    int iYSftMotAng : 9;				//实际换挡臂Y电机转角
    int iXSftMotAng : 9;				//实际换挡臂X电机转角
}T_STATE_OUT_6EA;

typedef struct
{
    int iXSftMotSpdL : 10;			//实际换挡臂X电机转速
    int iXSftMotSpdH : 1;				//实际换挡臂X电机转速
    unsigned int : 21;
}T_STATE_OUT_6EA_SRC;

typedef struct
{
    int iXSftMotSpd : 11;				//实际换挡臂X电机转速
    unsigned int : 21;
}T_STATE_OUT_6EA_DEST;

typedef union
{
    T_STATE_OUT_6EA_SRC tStateOut6EASrc;
    T_STATE_OUT_6EA_DEST tStateOut6EADest;
}T_STATE_OUT_6EA_UNION;

//档位控制状态反馈
typedef struct
{
    int iXSftMotAng;			//实际换挡臂X电机转角
    int iYSftMotAng;			//实际换挡臂Y电机转角
    int iXSftMotTrq;			//计算换挡臂X电机转矩
    int iYSftMotTrq;			//计算换挡臂Y电机转矩
    int iXSftMotSpd;			//实际换挡臂X电机转速
    int iYSftMotSpd;			//实际换挡臂Y电机转速
    unsigned int uiRbtSftPsng;	//机器人控制的挡位位置
    unsigned int uiSftSpdType;	//换挡完成类型
}T_SFT_STATUS;

#define	RBT_SFT_PSNG_BETWEEN		0	//机器人控制的挡位位置-???????????
#define	RBT_SFT_PSNG_PARK			1	//机器人控制的挡位位置-停车档（自动档）
#define	RBT_SFT_PSNG_REVERSE		2	//机器人控制的挡位位置-倒档
#define	RBT_SFT_PSNG_NEUTRAL		3	//机器人控制的挡位位置-空档
#define	RBT_SFT_PSNG_FORWARD_1		4	//机器人控制的挡位位置-前进档1(手动档)
#define	RBT_SFT_PSNG_FORWARD_2		5	//机器人控制的挡位位置-前进档2(手动档)
#define	RBT_SFT_PSNG_FORWARD_3		6	//机器人控制的挡位位置-前进档3(手动档)
#define	RBT_SFT_PSNG_FORWARD_4		7	//机器人控制的挡位位置-前进档4(手动档)
#define	RBT_SFT_PSNG_FORWARD_5		8	//机器人控制的挡位位置-前进档5(手动档)
#define	RBT_SFT_PSNG_FORWARD_6		9	//机器人控制的挡位位置-前进档6(手动档)
#define	RBT_SFT_PSNG_FORWARD_7		10	//机器人控制的挡位位置-前进档7(手动档)
#define	RBT_SFT_PSNG_FORWARD_8		11	//机器人控制的挡位位置-前进档8(手动档)
#define	RBT_SFT_PSNG_DRIVE			12	//机器人控制的挡位位置-前进档（自动档）
#define	RBT_SFT_PSNG_ADD			13	//机器人控制的挡位位置-加
#define	RBT_SFT_PSNG_SUB			14	//机器人控制的挡位位置-减
#define	RBT_SFT_PSNG_UNKOWN			15	//机器人控制的挡位位置-未定义
#define	RBT_SFT_PSNG_SPORT			16	//机器人控制的挡位位置-运动档（自动档）

#define	STF_SPD_TYPE_SLOW		0	//换挡模式-平滑型
#define	STF_SPD_TYPE_FAST		1	//换挡模式-快速型
#define	STF_SPD_TYPE_FASTER		2	//换挡模式-激进型


/*************系统运行状态参数反馈1*************/
typedef struct
{
    unsigned int uiClchMotCur : 8;	//实际离合电机电流
    unsigned int uiYSftMotCur : 8;	//实际换挡臂Y电机电流
    unsigned int uiXSftMotCur : 8;	//实际换挡臂X电机电流
    unsigned int uiStrMotCur : 8;		//实际转向电机电流
    unsigned int uiBrkMotCur : 8;		//实际制动电机电流
    unsigned int uiAccMotCur : 8;		//实际油门踏板电机电流
    unsigned int : 5;
    unsigned int uiRbtRunTime : 11;	//系统运行时间
}T_STATE_OUT_6EB;
// 系统运行状态参数反馈1
typedef struct
{
    unsigned int uiRbtRunTime;		//系统运行时间
    unsigned int uiAccMotCur;		//实际油门踏板电机电流
    unsigned int uiBrkMotCur;		//实际制动电机电流
    unsigned int uiStrMotCur;		//实际转向电机电流
    unsigned int uiXSftMotCur;		//实际换挡臂X电机电流
    unsigned int uiYSftMotCur;		//实际换挡臂Y电机电流
    unsigned int uiClchMotCur;		//实际离合电机电流
}T_SYS_INFO_1;


/*************系统运行状态参数反馈?????*************/
typedef struct
{
    int a;//实际各电机电压
    int b;
}T_STATE_OUT_6EC;


/*************系统运行状态参数反馈2******技术规范修改了定义*******/
typedef struct
{
    unsigned int uiYSftArmCtrAlive : 4;	//换挡臂Y控制器心跳
    unsigned int : 3;
    unsigned int uiYSftMotClbr : 1;		//换挡臂Y控制标零状态
    unsigned int uiXSftArmCtrAlive : 4;	//换挡臂X控制器心跳
    unsigned int : 3;
    unsigned int uiXSftMotClbr : 1;		//换挡臂X控制标零状态
    unsigned int uiStrCtrAlive : 4;		//转向控制器心跳
    unsigned int : 3;
    unsigned int uiStrMotClbr : 1;		//转向控制电机标零状态
    unsigned int uiClchCtrAlive : 4;		//离合控制器心跳
    unsigned int : 3;
    unsigned int uiClchMotClbr : 1;		//离合控制电机标零状态
    unsigned int uiBrkCtrAlive : 4;		//制动控制器心跳
    unsigned int : 3;
    unsigned int uiBrkMotClbr : 1;		//制动控制电机标零状态
    unsigned int uiAccCtrAlive : 4;		//油门踏板控制器心跳
    unsigned int : 3;
    unsigned int uiAccMotClbr : 1;		//油门踏板控制电机标零状态
    unsigned int uiRCUAlive : 4;			//系统RCU心跳
    unsigned int : 4;
    unsigned int uiCtrMdStat : 3;			//实际系统控制状态
    unsigned int : 5;
}T_STATE_OUT_6ED;

//系统运行状态参数反馈2
typedef struct
{
    unsigned int uiCtrMdStat;		//实际系统控制状态
    unsigned int uiRCUAlive;		//系统RCU心跳
    unsigned int uiAccCtrAlive;		//油门踏板控制器心跳
    unsigned int uiAccMotClbr;		//油门踏板控制电机标零状态
    unsigned int uiBrkCtrAlive;		//制动控制器心跳
    unsigned int uiBrkMotClbr;		//制动控制电机标零状态
    unsigned int uiClchCtrAlive;	//离合控制器心跳
    unsigned int uiClchMotClbr;		//离合控制电机标零状态
    unsigned int uiStrCtrAlive;		//转向控制器心跳
    unsigned int uiStrMotClbr;		//转向控制电机标零状态
    unsigned int uiXSftArmCtrAlive;	//换挡臂X控制器心跳
    unsigned int uiXSftMotClbr;		//换挡臂X控制标零状态
    unsigned int uiYSftArmCtrAlive;	//换挡臂Y控制器心跳
    unsigned int uiYSftMotClbr;		//换挡臂Y控制标零状态
}T_SYS_INFO_2;

#define	CTR_MD_STAT_AUTO_LEARN		0	//实际系统控制状态-自动学习模式
#define	CTR_MD_STAT_HAND_LEARN		1	//实际系统控制状态-手动学习模式
#define	CTR_MD_STAT_AUTO_DRIVE		2	//实际系统控制状态-自动驾驶模式
#define	CTR_MD_STAT_HAND_DRIVE		3	//实际系统控制状态-人工驾驶模式
#define	CTR_MD_STAT_ROBOT_CHECK		4	//实际系统控制状态-机器人自检模式
#define	CTR_MD_STAT_ROBOT_DEBUG		5	//实际系统控制状态-机器人调试模式

#define	MOT_CLBR_NO			0		//未标零
#define	MOT_CLBR_YES		1		//已标零


/*************系统学习状态反馈******技术规范修改了定义*******/
typedef struct
{

    unsigned int uiClch0LrnStat : 2;	//离合零位学习状态
    unsigned int uiClchMaxLrnStat : 2;//离合最大值学习状态
    unsigned int : 4;
    unsigned int uiStr0LrnStat : 2;	//转向零位学习状态
    unsigned int uiStrLLrnStat : 2;	//转向左极限学习状态
    unsigned int uiStrRLrnStat : 2;	//转向右极限学习状态
    unsigned int : 2;
    unsigned int uiAcc0LrnStat : 2;	//油门零位学习状态
    unsigned int uiAccMaxLrnStat : 2;	//油门最大值学习状态
    unsigned int uiBrk0LrnStat : 2;	//制动零位学习状态
    unsigned int uiBrkMaxLrnStat : 2;	//制动最大值学习状态
    unsigned int uiMLrnStat : 2;		//M档学习状态
    unsigned int uiUpLrnStat : 2;		//升学习状态
    unsigned int uiDpLrnStat : 2;		//降学习状态
    unsigned int : 2;
    unsigned int uiPLrnStat : 2;		//P档学习状态
    unsigned int uiNLrnStat : 2;		//N档学习状态
    unsigned int uiRLrnStat : 2;		//R档学习状态
    unsigned int uiDLrnStat : 2;		//D档学习状态
    unsigned int uiSftLrnStat : 2;	//挡位学习状态
    unsigned int : 6;
    unsigned int uiClchLrnStat : 2;	//离合学习状态
    unsigned int : 2;
    unsigned int uiStrLrnStat : 2;	//转向学习状态
    unsigned int : 2;
    unsigned int uiAccLrnStat : 2;	//油门学习状态
    unsigned int : 2;
    unsigned int uiBrkLrnStat : 2;	//制动学习状态
    unsigned int : 2;
}T_STATE_OUT_6EE;

//系统学习状态反馈
typedef struct
{
    unsigned int uiAccLrnStat;		//油门学习状态
    unsigned int uiBrkLrnStat;		//制动学习状态
    unsigned int uiClchLrnStat;		//离合学习状态
    unsigned int uiStrLrnStat;		//转向学习状态
    unsigned int uiSftLrnStat;		//挡位学习状态
    unsigned int uiPLrnStat;		//P档学习状态
    unsigned int uiNLrnStat;		//N档学习状态
    unsigned int uiRLrnStat;		//R档学习状态
    unsigned int uiDLrnStat;		//D档学习状态
    unsigned int uiMLrnStat;		//M档学习状态
    unsigned int uiUpLrnStat;		//升学习状态
    unsigned int uiDpLrnStat;		//降学习状态
    unsigned int uiAcc0LrnStat;		//油门零位学习状态
    unsigned int uiAccMaxLrnStat;	//油门最大值学习状态
    unsigned int uiBrk0LrnStat;		//制动零位学习状态
    unsigned int uiBrkMaxLrnStat;	//制动最大值学习状态
    unsigned int uiClch0LrnStat;	//离合零位学习状态
    unsigned int uiClchMaxLrnStat;	//离合最大值学习状态
    unsigned int uiStr0LrnStat;		//转向零位学习状态
    unsigned int uiStrLLrnStat;		//转向左极限学习状态
    unsigned int uiStrRLrnStat;		//转向右极限学习状态
}T_SYS_LRN;

#define	LRN_STAT_NOT			0	//未学习
#define	LRN_STAT_ING			1	//学习中
#define	LRN_STAT_COMPLETE		2	//学习完成
#define	LRN_STAT_TERMINATION	3	//学习终止


#define	SLFCHK_NOT				0	//自检未开始
#define	SLFCHK_ING				1	//自检进行中
#define	SLFCHK_COMPLETE			2	//自检通过
#define	SLFCHK_FAULT			3	//自检失败
#define	SLFCHK_TERMINATION		4	//自检终止

/*************档位学习位置反馈2******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 1;
    int iDownYMotPst : 9;		//降挡Y电机位置
    int iDownXMotPst : 9;		//降挡X电机位置
    int iUpYMotPst : 9;		//升挡Y电机位置
    int iUpXMotPstL : 4;		//升挡X电机位置
    int iUpXMotPstH : 5;		//升挡X电机位置
    int iMYMotPst : 9;		//M挡Y电机位置
    int iMXMotPst : 9;		//M挡X电机位置
    int iDYMotPst : 9;		//D挡Y电机位置
}T_STATE_OUT_6EF;

typedef struct
{
    int iUpXMotPstL : 4;		//升挡X电机位置
    int iUpXMotPstH : 5;		//升挡X电机位置
    unsigned int : 23;
}T_STATE_OUT_6EF_SRC;

typedef struct
{
    int iUpXMotPst : 9;		//升挡X电机位置
    unsigned int : 23;
}T_STATE_OUT_6EF_DEST;

typedef union
{
    T_STATE_OUT_6EF_SRC tStateOut6EFSrc;
    T_STATE_OUT_6EF_DEST tStateOut6EFDest;
}T_STATE_OUT_6EF_UNION;

//档位学习位置反馈2
typedef struct
{
    int iDYMotPst;		//D挡Y电机位置
    int iMXMotPst;		//M挡X电机位置
    int iMYMotPst;		//M挡Y电机位置
    int iUpXMotPst;		//升挡X电机位置
    int iUpYMotPst;		//升挡Y电机位置
    int iDownXMotPst;	//降挡X电机位置
    int iDownYMotPst;	//降挡Y电机位置
}T_SFT_LRN_PST_2;


/*************系统监控状态反馈******技术规范修改了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int uiCtrHum : 7;			//控制器环境湿度
    unsigned int : 1;
    int iCtrTmp : 8;						//控制器环境温度
    unsigned int uiRCUSftwAbnStat : 2;	//RCU控制软件状态
    unsigned int : 2;
    unsigned int uiRbtSlfChkStat : 3;		//机器人自检反馈
    unsigned int : 1;
    unsigned int uiClchMotAbnStat : 1;	//离合电机异常状态
    unsigned int uiClchCtrAbnStat : 1;	//离合控制器异常状态
    unsigned int uiXSftMotAbnStat : 1;	//换挡臂X电机异常状态
    unsigned int uiYSftMotAbnStat : 1;	//换挡臂Y电机异常状态
    unsigned int uiXSftArmCtrAbnStat : 1;	//换挡臂X电机控制器异常状态
    unsigned int uiYSftArmCtrAbnStat : 1;	//换挡臂Y电机控制器异常状态
    unsigned int uiRCUHdwAbnStat : 1;		//RCU控制器硬件异常状态
    unsigned int : 1;
    unsigned int uiSysAbnStat : 1;		//系统异常状态
    unsigned int uiAccMotAbnStat : 1;		//油门踏板电机异常状态
    unsigned int uiAccCtrAbnStat : 1;		//油门踏板控制器异常状态
    unsigned int uiAccClchAbnStat : 1;	//油门踏板离合器异常状态
    unsigned int uiBrkMotAbnStat : 1;		//制动电机异常状态
    unsigned int uiBrkCtrAbnStat : 1;		//制动控制器异常状态
    unsigned int uiStrMotAbnStat : 1;		//转向电机异常状态
    unsigned int uiStrCtrAbnStat : 1;		//转向控制器异常状态
}T_STATE_OUT_6D0;

//系统监控状态反馈
typedef struct
{
    unsigned int uiSysAbnStat;			//系统异常状态
    unsigned int uiAccMotAbnStat;		//油门踏板电机异常状态
    unsigned int uiAccCtrAbnStat;		//油门踏板控制器异常状态
    unsigned int uiAccClchAbnStat;		//油门踏板离合器异常状态
    unsigned int uiBrkMotAbnStat;		//制动电机异常状态
    unsigned int uiBrkCtrAbnStat;		//制动控制器异常状态
    unsigned int uiStrMotAbnStat;		//转向电机异常状态
    unsigned int uiStrCtrAbnStat;		//转向控制器异常状态
    unsigned int uiClchMotAbnStat;		//离合电机异常状态
    unsigned int uiClchCtrAbnStat;		//离合控制器异常状态
    unsigned int uiXSftMotAbnStat;		//换挡臂X电机异常状态
    unsigned int uiYSftMotAbnStat;		//换挡臂Y电机异常状态
    unsigned int uiXSftArmCtrAbnStat;	//换挡臂X电机控制器异常状态
    unsigned int uiYSftArmCtrAbnStat;	//换挡臂Y电机控制器异常状态
    unsigned int uiRCUHdwAbnStat;		//RCU控制器硬件异常状态
    unsigned int uiRCUSftwAbnStat;		//RCU控制软件状态
    unsigned int uiRbtSlfChkStat;		//机器人自检反馈
    int iCtrTmp;						//控制器环境温度
    unsigned int uiCtrHum;				//控制器环境湿度
}T_SYS_MONITOR;

#define	ABN_STAT_NO_ERROR		0		//无异常
#define	ABN_STAT_ERROR			1		//异常

#define	RCU_SFTW_ABN_STAT_STOP		0	//stop
#define	RCU_SFTW_ABN_STAT_RUN		1	//run
#define	RCU_SFTW_ABN_STAT_ERROR		2	//error

#define	RBT_SLF_CHK_STAT_NO_START		0	//自检未开始
#define	RBT_SLF_CHK_STAT_ING			1	//自检进行中
#define	RBT_SLF_CHK_STAT_OK				2	//自检通过
#define	RBT_SLF_CHK_STAT_FAILED			3	//自检失败
#define	RBT_SLF_CHK_STAT_TERMINATION	4	//自检终止


/************系统报警******技术规范修改了定义*******/
typedef struct
{
    unsigned int : 32;
    unsigned int uiSysErry : 1;
    unsigned int uiSysErrz : 1;
    unsigned int : 6;
    unsigned int uiSysErrq : 1;
    unsigned int uiSysErrr : 1;
    unsigned int uiSysErrs : 1;
    unsigned int uiSysErrt : 1;
    unsigned int uiSysErru : 1;
    unsigned int uiSysErrv : 1;
    unsigned int uiSysErrw : 1;
    unsigned int uiSysErrx : 1;
    unsigned int uiSysErri : 1;
    unsigned int uiSysErrj : 1;
    unsigned int uiSysErrk : 1;
    unsigned int uiSysErrl : 1;
    unsigned int uiSysErrm : 1;
    unsigned int uiSysErrn : 1;
    unsigned int uiSysErro : 1;
    unsigned int uiSysErrp : 1;
    unsigned int uiSysErra : 1;
    unsigned int uiSysErrb : 1;
    unsigned int uiSysErrc : 1;
    unsigned int uiSysErrd : 1;
    unsigned int uiSysErre : 1;
    unsigned int uiSysErrf : 1;
    unsigned int uiSysErrg : 1;
    unsigned int uiSysErrh : 1;
}T_STATE_OUT_6D1;

typedef struct
{
    unsigned int uiSysErra : 1;
    unsigned int uiSysErrb : 1;
    unsigned int uiSysErrc : 1;
    unsigned int uiSysErrd : 1;
    unsigned int uiSysErre : 1;
    unsigned int uiSysErrf : 1;
    unsigned int uiSysErrg : 1;
    unsigned int uiSysErrh : 1;
    unsigned int uiSysErri : 1;
    unsigned int uiSysErrj : 1;
    unsigned int uiSysErrk : 1;
    unsigned int uiSysErrl : 1;
    unsigned int uiSysErrm : 1;
    unsigned int uiSysErrn : 1;
    unsigned int uiSysErro : 1;
    unsigned int uiSysErrp : 1;
    unsigned int uiSysErrq : 1;
    unsigned int uiSysErrr : 1;
    unsigned int uiSysErrs : 1;
    unsigned int uiSysErrt : 1;
    unsigned int uiSysErru : 1;
    unsigned int uiSysErrv : 1;
    unsigned int uiSysErrw : 1;
    unsigned int uiSysErrx : 1;
    unsigned int uiSysErry : 1;
    unsigned int uiSysErrz : 1;
    unsigned int : 5;
    unsigned int : 32;
}T_SYS_ERR;

/************油门报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiAccErrag : 1;
    unsigned int  uiAccErrah : 1;
    unsigned int  uiAccErrai : 1;
    unsigned int : 5;
    unsigned int  uiAccErry : 1;
    unsigned int  uiAccErrz : 1;
    unsigned int  uiAccErraa : 1;
    unsigned int  uiAccErrab : 1;
    unsigned int  uiAccErrac : 1;
    unsigned int  uiAccErrad : 1;
    unsigned int  uiAccErrae : 1;
    unsigned int  uiAccErraf : 1;
    unsigned int  uiAccErrq : 1;
    unsigned int  uiAccErrr : 1;
    unsigned int  uiAccErrs : 1;
    unsigned int  uiAccErrt : 1;
    unsigned int  uiAccErru : 1;
    unsigned int  uiAccErrv : 1;
    unsigned int  uiAccErrw : 1;
    unsigned int  uiAccErrx : 1;
    unsigned int  uiAccErri : 1;
    unsigned int  uiAccErrj : 1;
    unsigned int  uiAccErrk : 1;
    unsigned int  uiAccErrl : 1;
    unsigned int  uiAccErrm : 1;
    unsigned int  uiAccErrn : 1;
    unsigned int  uiAccErro : 1;
    unsigned int  uiAccErrp : 1;
    unsigned int  uiAccErra : 1;
    unsigned int  uiAccErrb : 1;
    unsigned int  uiAccErrc : 1;
    unsigned int  uiAccErrd : 1;
    unsigned int  uiAccErre : 1;
    unsigned int  uiAccErrf : 1;
    unsigned int  uiAccErrg : 1;
    unsigned int  uiAccErrh : 1;
}T_ACC_ERR;

/************制动报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiBrkErrag : 1;
    unsigned int  uiBrkErrah : 1;
    unsigned int  uiBrkErrai : 1;
    unsigned int : 5;
    unsigned int  uiBrkErry : 1;
    unsigned int  uiBrkErrz : 1;
    unsigned int  uiBrkErraa : 1;
    unsigned int  uiBrkErrab : 1;
    unsigned int  uiBrkErrac : 1;
    unsigned int  uiBrkErrad : 1;
    unsigned int  uiBrkErrae : 1;
    unsigned int  uiBrkErraf : 1;
    unsigned int  uiBrkErrq : 1;
    unsigned int  uiBrkErrr : 1;
    unsigned int  uiBrkErrs : 1;
    unsigned int  uiBrkErrt : 1;
    unsigned int  uiBrkErru : 1;
    unsigned int  uiBrkErrv : 1;
    unsigned int  uiBrkErrw : 1;
    unsigned int  uiBrkErrx : 1;
    unsigned int  uiBrkErri : 1;
    unsigned int  uiBrkErrj : 1;
    unsigned int  uiBrkErrk : 1;
    unsigned int  uiBrkErrl : 1;
    unsigned int  uiBrkErrm : 1;
    unsigned int  uiBrkErrn : 1;
    unsigned int  uiBrkErro : 1;
    unsigned int  uiBrkErrp : 1;
    unsigned int  uiBrkErra : 1;
    unsigned int  uiBrkErrb : 1;
    unsigned int  uiBrkErrc : 1;
    unsigned int  uiBrkErrd : 1;
    unsigned int  uiBrkErre : 1;
    unsigned int  uiBrkErrf : 1;
    unsigned int  uiBrkErrg : 1;
    unsigned int  uiBrkErrh : 1;
}T_BRK_ERR;

/************档位X电机报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiXSftErrag : 1;
    unsigned int  uiXSftErrah : 1;
    unsigned int  uiXSftErrai : 1;
    unsigned int : 5;
    unsigned int  uiXSftErry : 1;
    unsigned int  uiXSftErrz : 1;
    unsigned int  uiXSftErraa : 1;
    unsigned int  uiXSftErrab : 1;
    unsigned int  uiXSftErrac : 1;
    unsigned int  uiXSftErrad : 1;
    unsigned int  uiXSftErrae : 1;
    unsigned int  uiXSftErraf : 1;
    unsigned int  uiXSftErrq : 1;
    unsigned int  uiXSftErrr : 1;
    unsigned int  uiXSftErrs : 1;
    unsigned int  uiXSftErrt : 1;
    unsigned int  uiXSftErru : 1;
    unsigned int  uiXSftErrv : 1;
    unsigned int  uiXSftErrw : 1;
    unsigned int  uiXSftErrx : 1;
    unsigned int  uiXSftErri : 1;
    unsigned int  uiXSftErrj : 1;
    unsigned int  uiXSftErrk : 1;
    unsigned int  uiXSftErrl : 1;
    unsigned int  uiXSftErrm : 1;
    unsigned int  uiXSftErrn : 1;
    unsigned int  uiXSftErro : 1;
    unsigned int  uiXSftErrp : 1;
    unsigned int  uiXSftErra : 1;
    unsigned int  uiXSftErrb : 1;
    unsigned int  uiXSftErrc : 1;
    unsigned int  uiXSftErrd : 1;
    unsigned int  uiXSftErre : 1;
    unsigned int  uiXSftErrf : 1;
    unsigned int  uiXSftErrg : 1;
    unsigned int  uiXSftErrh : 1;
}T_XSFT_ERR;

/************档位Y电机报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiYSftErrag : 1;
    unsigned int  uiYSftErrah : 1;
    unsigned int  uiYSftErrai : 1;
    unsigned int : 5;
    unsigned int  uiYSftErry : 1;
    unsigned int  uiYSftErrz : 1;
    unsigned int  uiYSftErraa : 1;
    unsigned int  uiYSftErrab : 1;
    unsigned int  uiYSftErrac : 1;
    unsigned int  uiYSftErrad : 1;
    unsigned int  uiYSftErrae : 1;
    unsigned int  uiYSftErraf : 1;
    unsigned int  uiYSftErrq : 1;
    unsigned int  uiYSftErrr : 1;
    unsigned int  uiYSftErrs : 1;
    unsigned int  uiYSftErrt : 1;
    unsigned int  uiYSftErru : 1;
    unsigned int  uiYSftErrv : 1;
    unsigned int  uiYSftErrw : 1;
    unsigned int  uiYSftErrx : 1;
    unsigned int  uiYSftErri : 1;
    unsigned int  uiYSftErrj : 1;
    unsigned int  uiYSftErrk : 1;
    unsigned int  uiYSftErrl : 1;
    unsigned int  uiYSftErrm : 1;
    unsigned int  uiYSftErrn : 1;
    unsigned int  uiYSftErro : 1;
    unsigned int  uiYSftErrp : 1;
    unsigned int  uiYSftErra : 1;
    unsigned int  uiYSftErrb : 1;
    unsigned int  uiYSftErrc : 1;
    unsigned int  uiYSftErrd : 1;
    unsigned int  uiYSftErre : 1;
    unsigned int  uiYSftErrf : 1;
    unsigned int  uiYSftErrg : 1;
    unsigned int  uiYSftErrh : 1;
}T_YSFT_ERR;

/************转向电机报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiStrErrag : 1;
    unsigned int  uiStrErrah : 1;
    unsigned int  uiStrErrai : 1;
    unsigned int : 5;
    unsigned int  uiStrErry : 1;
    unsigned int  uiStrErrz : 1;
    unsigned int  uiStrErraa : 1;
    unsigned int  uiStrErrab : 1;
    unsigned int  uiStrErrac : 1;
    unsigned int  uiStrErrad : 1;
    unsigned int  uiStrErrae : 1;
    unsigned int  uiStrErraf : 1;
    unsigned int  uiStrErrq : 1;
    unsigned int  uiStrErrr : 1;
    unsigned int  uiStrErrs : 1;
    unsigned int  uiStrErrt : 1;
    unsigned int  uiStrErru : 1;
    unsigned int  uiStrErrv : 1;
    unsigned int  uiStrErrw : 1;
    unsigned int  uiStrErrx : 1;
    unsigned int  uiStrErri : 1;
    unsigned int  uiStrErrj : 1;
    unsigned int  uiStrErrk : 1;
    unsigned int  uiStrErrl : 1;
    unsigned int  uiStrErrm : 1;
    unsigned int  uiStrErrn : 1;
    unsigned int  uiStrErro : 1;
    unsigned int  uiStrErrp : 1;
    unsigned int  uiStrErra : 1;
    unsigned int  uiStrErrb : 1;
    unsigned int  uiStrErrc : 1;
    unsigned int  uiStrErrd : 1;
    unsigned int  uiStrErre : 1;
    unsigned int  uiStrErrf : 1;
    unsigned int  uiStrErrg : 1;
    unsigned int  uiStrErrh : 1;
}T_STR_ERR;

/************油门(acc)、制动(brk)、挡位(sft)X、挡位(sft)Y、转向(str)电机报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  ag : 1;
    unsigned int  ah : 1;
    unsigned int  ai : 1;
    unsigned int : 5;
    unsigned int  y : 1;
    unsigned int  z : 1;
    unsigned int  aa : 1;
    unsigned int  ab : 1;
    unsigned int  ac : 1;
    unsigned int  ad : 1;
    unsigned int  ae : 1;
    unsigned int  af : 1;
    unsigned int  q : 1;
    unsigned int  r : 1;
    unsigned int  s : 1;
    unsigned int  t : 1;
    unsigned int  u : 1;
    unsigned int  v : 1;
    unsigned int  w : 1;
    unsigned int  x : 1;
    unsigned int  i : 1;
    unsigned int  j : 1;
    unsigned int  k : 1;
    unsigned int  l : 1;
    unsigned int  m : 1;
    unsigned int  n : 1;
    unsigned int  o : 1;
    unsigned int  p : 1;
    unsigned int  a : 1;
    unsigned int  b : 1;
    unsigned int  c : 1;
    unsigned int  d : 1;
    unsigned int  e : 1;
    unsigned int  f : 1;
    unsigned int  g : 1;
    unsigned int  h : 1;
}T_CTR_ERR_CODE;

//油门(acc)、制动(brk)、挡位(sft)X、挡位(sft)Y、转向(str)电机报警
typedef struct
{
    unsigned int  uiErrag;
    unsigned int  uiErrah;
    unsigned int  uiErrai;
    unsigned int  uiErry;
    unsigned int  uiErrz;
    unsigned int  uiErraa;
    unsigned int  uiErrab;
    unsigned int  uiErrac;
    unsigned int  uiErrad;
    unsigned int  uiErrae;
    unsigned int  uiErraf;
    unsigned int  uiErrq;
    unsigned int  uiErrr;
    unsigned int  uiErrs;
    unsigned int  uiErrt;
    unsigned int  uiErru;
    unsigned int  uiErrv;
    unsigned int  uiErrw;
    unsigned int  uiErrx;
    unsigned int  uiErri;
    unsigned int  uiErrj;
    unsigned int  uiErrk;
    unsigned int  uiErrl;
    unsigned int  uiErrm;
    unsigned int  uiErrn;
    unsigned int  uiErro;
    unsigned int  uiErrp;
    unsigned int  uiErra;
    unsigned int  uiErrb;
    unsigned int  uiErrc;
    unsigned int  uiErrd;
    unsigned int  uiErre;
    unsigned int  uiErrf;
    unsigned int  uiErrg;
    unsigned int  uiErrh;
}T_MOT_CTR_ERR;

/************方向盘位置时间控制请求*************/
typedef struct
{
    unsigned int : 32;
    unsigned int : 16;
    unsigned int uiNumReq : 8;  //请求实际点序号
    unsigned int uiSeqIdReq : 8;   //	请求序列ID
}T_CONTROL_CMD_6D7;

//方向盘位置时间控制请求
typedef struct
{
    unsigned int uiNumReq;  //请求实际点序号
    unsigned int uiSeqIdReq;   //	请求序列ID
}T_STEER_PST_TIME_REQ;

/******************************ESTOP状态信息******************************/
/************Estop系统信息******技术规范修改了定义*******/
typedef struct
{
    unsigned int : 17;
    unsigned int uiEstpAbnStat : 1;	//Estop异常状态
    unsigned int : 14;
    unsigned int : 8;
    unsigned int uiPmpEnStat : 1;		//急停制动气泵使能状态
    unsigned int : 7;
    unsigned int uiEstpEnStat : 1;	//机器人Estop使能状态
    unsigned int : 7;
    unsigned int uiEstpAlive : 4;		//Estop心跳
    unsigned int : 4;
}T_ESTOP_2E0;

typedef struct
{
    unsigned int uiEstpAlive;		//Estop心跳
    unsigned int uiEstpEnStat;		//机器人Estop使能状态
    unsigned int uiPmpEnStat;		//急停制动气泵使能状态
    unsigned int uiEstpAbnStat;		//Estop异常状态
}T_ESTOP_INFO;

#define ESTP_EN_STAT_DISABLE	0	//机器人Estop使能状态-禁能
#define ESTP_EN_STAT_ENABLE		1	//机器人Estop使能状态-使能

#define PMP_EN_STAT_DISABLE		0	//急停制动气泵使能状态-禁能
#define PMP_EN_STAT_ENABLE		1	//急停制动气泵使能状态-使能

#define ESTP_ABN_STAT_NO_ERROR	0	//Estop异常状态-无异常
#define ESTP_ABN_STAT_ERROR		1	//Estop异常状态-异常


/************Estop故障码******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 32;
    unsigned int : 16;
    unsigned int uiEspErri : 1;
    unsigned int uiEspErrj : 1;
    unsigned int : 6;
    unsigned int uiEspErra : 1;
    unsigned int uiEspErrb : 1;
    unsigned int uiEspErrc : 1;
    unsigned int uiEspErrd : 1;
    unsigned int uiEspErre : 1;
    unsigned int uiEspErrf : 1;
    unsigned int uiEspErrg : 1;
    unsigned int uiEspErrh : 1;
}T_ESTOP_2E1;

typedef struct
{
    unsigned int uiEspErra : 1;
    unsigned int uiEspErrb : 1;
    unsigned int uiEspErrc : 1;
    unsigned int uiEspErrd : 1;
    unsigned int uiEspErre : 1;
    unsigned int uiEspErrf : 1;
    unsigned int uiEspErrg : 1;
    unsigned int uiEspErrh : 1;
    unsigned int uiEspErri : 1;
    unsigned int uiEspErrj : 1;
    unsigned int : 22;
}T_ESTOP_ERR;

typedef struct
{
    unsigned int uiVehSpd : 16;
    unsigned int : 16;
    unsigned int : 32;
}T_STATE_IN_603;

typedef struct
{
    unsigned int uiVehSpd;
}T_RTK_SPEED;



/************OBD实际油门踏板行程*******/
typedef struct
{
    unsigned int : 32;
    unsigned int : 16;
    unsigned int obdAccPsng : 8;		//实际油门踏板行程
    unsigned int : 8;
}T_OBD_1A1;

//实际油门踏板行程
typedef struct
{
    unsigned int obdAccPsng;			//实际油门踏板行程
}T_OBD_ACC_PSNG;

#define OBD_ACC_PSNG_RATIO		255	//实际油门踏板行程比例

/************OBD实际制动踏板行程*******/
typedef struct
{
    unsigned int : 8;
    unsigned int obdBrkPsng : 8;		//实际制动踏板行程
    unsigned int : 16;
    unsigned int : 32;
}T_OBD_0F1;

//实际制动踏板行程
typedef struct
{
    unsigned int obdBrkPsng;			//实际制动踏板行程
}T_OBD_BRK_PSNG;

#define OBD_BRK_PSNG_RATIO		255	//实际制动踏板行程比例

/************OBD实际挡位*******/
typedef struct
{
    unsigned int obdSftPsng : 8;		//实际挡位
    unsigned int : 24;
    unsigned int : 32;
}T_OBD_135;

//实际挡位
typedef struct
{
    unsigned int obdSftPsng;			//实际挡位
}T_OBD_SFT_PSNG;

#define SFT_BETWEEN_RANGE		0	//Between Range
#define SFT_PARK_RANGE		    1	//Park Range
#define SFT_REVERSE_RANGE		2	//Reverse Range
#define SFT_NEUTRAL_RANGE		3	//Neutral Range
#define SFT_DRIVE_RANGE	    	4	//Drive Range

/************OBD车辆手刹状态*******/
typedef struct
{
    unsigned int : 32;
    unsigned int obdVehHBrkStatFix: 4;    //车辆手刹状态固定位
    unsigned int obdVehHBrkStat : 4;	 	//车辆手刹状态
    unsigned int : 24;
}T_OBD_1F1;

//车辆手刹状态
typedef struct
{
    unsigned int obdVehHBrkStatFix;    //车辆手刹状态固定位
    unsigned int obdVehHBrkStat;			//车辆手刹状态

}T_OBD_VEH_BRK_STAT;

/************OBD实际车辆启动状态*******/
typedef struct
{
    unsigned int obdVehSwStat : 8;		//实际车辆启动状态
    unsigned int : 24;
    unsigned int : 32;
}T_OBD_121;

//实际车辆启动状态
typedef struct
{
    unsigned int obdVehSwStat;			//实际车辆启动状态
}T_OBD_VEH_SW_STAT;

#define VEH_LOCK     0  //LOCK熄火
#define VEH_ACC      1  //ACC部分上电
#define VEH_ON       2  //ON已启动
#define VEH_START    3  //START打火

/************OBD实际方向盘转角*******/
typedef struct
{
    unsigned int : 8;
    int obdStrAngH : 8;		//实际方向盘转角
    int obdStrAngL : 8;		//实际方向盘转角
    unsigned int : 8;
    unsigned int : 32;
}T_OBD_1E5;

typedef struct
{
    int obdStrAngL : 8;		//实际方向盘转角
    int obdStrAngH : 8;		//实际方向盘转角
    unsigned int : 16;
}T_OBD_1E5_SRC;

typedef struct
{
    int obdStrAng : 16;		//实际方向盘转角
    unsigned int : 16;
}T_OBD_1E5_DEST;

typedef union
{
    T_OBD_1E5_SRC tobd1e5Src;		//实际方向盘转角
    T_OBD_1E5_DEST  tobd1e5Dest;
}T_OBD_1E5_UNION;

//实际方向盘转角
typedef struct
{
    int obdStrAng;			//实际方向盘转角
}T_OBD_STR_ANG;

#define OBD_STR_ANG_RATIO		16	//实际方向盘转角比例

/************OBD实际发动机转速*******/
typedef struct
{
    unsigned int : 8;
    unsigned int obdEngSpd : 16;		//实际发动机转速
    unsigned int : 8;
    unsigned int : 32;
}T_OBD_0C9;

//实际发动机转速
typedef struct
{
    unsigned int obdEngSpd;			//实际发动机转速
}T_OBD_ENG_SPD;

#define OBD_ENG_SPD_RATIO		4	//实际发动机转速比例

} // namespace canbus

} // namespace robot


#endif /* CAN_DATA_TYPE_H_*/
