#include "decision.h"
#include "../common/message.h"
#include "decisiondatatype.h"
#include <strstream>
#include <string.h>
#include <canbus/PedalActCtrReq.h>
#include <canbus/SftActCtrReq.h>
#include <canbus/StrActCtrReq.h>
#include <decision/localizationMsg.h>
#include <decision/rulesMsg.h>
#include <decision/controlMsg.h>
#include <stdlib.h>
#include <math.h>


namespace robot {
namespace decisionSpace {



std::string Decision::Name() const { return "decision"; }

int Decision::Init(){

    LOG(INFO)<<Name()<<": init() start...";
    node_handle_.reset(new ros::NodeHandle());
    DecisionDatabase::InitDB();
    iCommandCmd = CMD_UNDEFIED;// set undefied mode
    iRulesTotalNumbers = ilocalTotalNumbers =fCarNowSpeed = iLocalNowNumber = iRulesNowNumber = 0;
    iPredictionPointsPerNumber = 5;
    uiCarMode =6; // undefied

    PlanRouteName = "siasun";
    ilocalTotalNumbers = 100;
    fCarNowSpeed = 0;
    vPx.push_back(127.162345678);
    vPy.push_back(41.843340742);
    vPx.push_back(127.162345678);
    vPy.push_back(41.843340742);
    vPx.push_back(127.162345678);
    vPy.push_back(41.843340742);
    vPx.push_back(127.162345678);
    vPy.push_back(41.843340742);
    vPx.push_back(127.162345678);
    vPy.push_back(41.843340742);
    vPx.push_back(127.162345678);
    vPy.push_back(41.843340742);
    vPx.push_back(127.160345678);
    vPy.push_back(41.843340742);
    vPx.push_back(127.154345678);
    vPy.push_back(41.851940742);
    vPx.push_back(127.154345678);
    vPy.push_back(41.851940742);
    vPx.push_back(127.154345678);
    vPy.push_back(41.851940742);
    vPx.push_back(127.154345678);
    vPy.push_back(41.851940742);
    for (int i =0; i< 11; i++)
    {
       TestMPC();
    }

    LOG(INFO)<<Name()<<" init() end.";
    return ROBOT_SUCCESS;

}

int Decision::Start(){
    LOG(INFO)<<Name()<<" start() start...";
    // under ros service server
    ros::ServiceServer CommandService = node_handle_->advertiseService(robot::common::CommandModeCmdService,
                                                                &Decision::CommandRequestService, this);
    // under ros subscriber
    ros::Subscriber GeometrySub = node_handle_->subscribe(robot::common::Geometry, 2000,
                                                          &Decision::RTKLocationCallback, this);

    ros::Subscriber OBDSub = node_handle_->subscribe(robot::common::ObdData, 2000, &Decision::OBDCallback, this);

    ros::Subscriber RCUModeSub = node_handle_->subscribe(robot::common::Si2Data, 2000, &Decision::RCUModeCallback, this);

    // under ros publish
    Rules_pub = node_handle_->advertise<decision::rulesMsg>(robot::common::RuleInfo, 200);

    Geometry_pub = node_handle_->advertise<decision::localizationMsg>(robot::common::GeometryPlan, 200);

    Control_pub = node_handle_->advertise<decision::controlMsg>(robot::common::ControlInfo, 200);

    // under ros service of client
    clientSftActCtrReq = node_handle_->serviceClient<canbus::SftActCtrReq>(robot::common::ShiftActCtrService);

    clientPedalActCtrReq = node_handle_->serviceClient<canbus::PedalActCtrReq>(robot::common::PedalActCtrService);

    clientStrActCtrReq = node_handle_->serviceClient<canbus::StrActCtrReq>(robot::common::SteerActCtrService);

    ros::AsyncSpinner spinner(2); // thread_count The number of threads to use.  A value of 0 means to use the number of processor cores
    spinner.start();
    ros::waitForShutdown();
    LOG(INFO)<<Name()<<"  start() end.";
    return ROBOT_SUCCESS;
}

void Decision::Stop(){
    LOG(INFO)<<Name()<<": stop start...";
      LOG(INFO)<<Name()<<": stop end.";
    return ;
}

//  执行自动驾驶模式和路径规划模式，自动驾驶模式踩刹车然后挂前进当；路径规划模式删除数据库记录
bool Decision::CommandRequestService(decision::CommandModeCmd::Request &req,
                      decision::CommandModeCmd::Response &res)
{
    LOG(INFO)<<Name()<<"CommandRequestService get command ID:."<< req.Cmd;
    if (req.Cmd == CMD_AUTODRIE_START)
    {
        LoadPlanRoute();
        double breakCtr = -5; // 踩刹车到50%
        ros::Rate loop_rate(1);
        while(ros::ok())
        {
            if (SpeedControl(breakCtr) == 1)
            {
                LOG(INFO)<<Name()<<"CommandRequestService break control 50% excute success.";
                if (BreakCtrControl(SFT_PSNG_LRN_REQ_FORWARD_1) == 1) // 前进挡
                    break;
            }
            else
            {
                LOG(ERROR)<<Name()<<"CommandRequestService break control 50% excute error.";
                return false;
            }
            ros::spinOnce();
            loop_rate.sleep();
        }
        fCarNowSpeed = iLocalNowNumber = iRulesNowNumber = 0;
    }
    else if (req.Cmd == CMD_PLAN_START)
    {
        DecisionDatabase::DeleteActionDirective(PlanRouteName.c_str());
        DecisionDatabase::DeleteLocationRuleInfo(PlanRouteName.c_str());
    }
    res.CmdResult = iCommandCmd = req.Cmd;
    LOG(INFO)<<Name()<<"CommandRequestService end."<< res.CmdResult;
    return true;
}


// 加载规划路径，同时把规划的路径轨迹和测试的用例发送给显示界面
void Decision::LoadPlanRoute()
{
    ilocalTotalNumbers = DecisionDatabase::countOfLocationRuleInfo();
    iRulesTotalNumbers = DecisionDatabase::countOfActionDirective();
    LOG(INFO)<<Name()<<" LoadPlanRoute select ilocalTotalNumbers: "<<ilocalTotalNumbers<<"  iRulesTotalNumbers: "<<iRulesTotalNumbers;
    std::vector<LOCATION_RULE_INFO> tempInfo = DecisionDatabase::QueryLocationRuleInfo(0, ilocalTotalNumbers);
    for (int i = 0; i< ilocalTotalNumbers; i++)
    {
        decision::localizationMsg localMsg;
        localMsg.Locationid = tempInfo[i].LocationID;
        localMsg.Longitude = tempInfo[i].LocationLon;
        localMsg.Latitude = tempInfo[i].LocationLat;
        localMsg.Height = tempInfo[i].LocationHeight;
        localMsg.LocationSpeed = tempInfo[i].LocationSpeed;
        Geometry_pub.publish(localMsg);
    }
    std::vector<ACTION_DIRECTIVE> tempAction = DecisionDatabase::QueryActionDirective(0, iRulesTotalNumbers);
    for (int i = 0; i< iRulesTotalNumbers; i++)
    {
        decision::rulesMsg ruleMsg;
        ruleMsg.DirectiveID = tempAction[i].DirectiveID;
        ruleMsg.DirectiveInfo = tempAction[i].DirectiveInfo;
        ruleMsg.DirectiveResult = tempAction[i].DirectiveResult;
        Rules_pub.publish(ruleMsg);
    }

}

// 接收ＲＴＫ数据分为自动驾驶模式开始和结束和路径规划模式开始和结束
void Decision::RTKLocationCallback(const localization::localMsg::ConstPtr& msg)
{
//     double secsStart =ros::Time::now().toSec();
     switch (iCommandCmd) {
     case CMD_PLAN_START:
         ExcutePlanRoute(msg);
         break;
     case CMD_AUTODRIE_START:
         ExcuteSelfDriving(msg);
         break;
     case CMD_PLAN_END:
         StopPlanRoute(msg);
         break;
     case CMD_AUTODRIE_END:
         CarAutoDriveStop();
         break;
     default:
         break;
     }
//    double secsEnd =ros::Time::now().toSec();
//    ros::Duration(2 - secsEnd + secsStart).sleep();
}


void Decision::OBDCallback(const canbus::obdMsg::ConstPtr& msg)
{
    fCarNowSpeed = msg->VehSpd;
    //LOG(INFO)<<Name()<< " car real speed:"<<fCarNowSpeed;
}

void Decision::RCUModeCallback(const perception::si2Msg::ConstPtr& msg)
{
   uiCarMode = msg->CtrMdStat;
   //LOG(INFO)<<Name()<< " car excute mode :"<<uiCarMode;
}

//自动生成测试用例
void Decision::ExcutePlanRoute(const localization::localMsg::ConstPtr& msg)
{
    double secsStart =ros::Time::now().toSec();
    try {
        iLocalNowNumber ++;
        LOG(INFO)<<Name()<<"ExcutePlanRoute iLocalNowNumber = "<< iLocalNowNumber;
        if ((iLocalNowNumber % iPredictionPointsPerNumber)  == 0) {          
            iRulesNowNumber++;
            std::stringstream strRuleContent;
            if (iRulesNowNumber == 1)
                strRuleContent<<"动作"<<iRulesNowNumber<<":起步请加速到"<< fCarNowSpeed<<"km/h";
            else{
                if (fCarNowSpeed > fCarLastSpeed)
                {
                    strRuleContent<<"动作"<<iRulesNowNumber<<":加速到"<< fCarNowSpeed<<"km/h";
                }
                else if (fCarNowSpeed < fCarLastSpeed)
                {
                    strRuleContent<<"动作"<<iRulesNowNumber<<":制动至"<< fCarNowSpeed<<"km/h";
                }
                else if (fCarNowSpeed == fCarLastSpeed)
                {
                    strRuleContent<<"动作"<<iRulesNowNumber<<":保持"<< fCarNowSpeed<<"km/h";
                }
           }

           int iRet = DecisionDatabase::InsertLocationRuleInfoAndActionDirective(PlanRouteName.c_str(), iRulesNowNumber,
                            strRuleContent.str().c_str(), RULE_EXCUTE_RESULT_INIT, iLocalNowNumber, msg->Longitude,
                            msg->Latitude, msg->Height, fCarNowSpeed);
           if (iRet < 0) {
                iLocalNowNumber--;
                iRulesNowNumber--;
           }
           fCarLastSpeed = fCarNowSpeed;
        }
        else{
           DecisionDatabase::InsertLocationRuleInfo(PlanRouteName.c_str(),iLocalNowNumber, msg->Longitude,
                                             msg->Latitude, msg->Height, fCarNowSpeed);
        }
    }
    catch (ros::Exception ex)
    {
        LOG(ERROR)<<Name()<<":ExcutePlanRoute error."<< ex.what();
    }
    double secsEnd =ros::Time::now().toSec();
    ros::Duration(2 - secsEnd + secsStart).sleep();
}

// 自动生成测试用例停止
void Decision::StopPlanRoute(const localization::localMsg::ConstPtr& msg)
{
     LOG(INFO)<<Name()<<"StopPlanRoute last record iLocalNowNumber = "<< iLocalNowNumber;
     iCommandCmd = CMD_UNDEFIED;// set undefied mode
     if ((iLocalNowNumber % iPredictionPointsPerNumber)  == 0){
          LOG(INFO)<<Name()<<"StopPlanRoute last record iLocalNowNumber = "<< iLocalNowNumber
                  << " iLocalNowNumber % iPredictionPointsPerNumber  = 0";
         return;
     }
     else{
         iLocalNowNumber++;
         iRulesNowNumber++;
         std::stringstream strRuleContent;
         if (fCarNowSpeed > fCarLastSpeed)
         {
             strRuleContent<<"动作"<<iRulesNowNumber<<":加速到"<< fCarNowSpeed<<"km/h";
         }
         else if (fCarNowSpeed < fCarLastSpeed)
         {
             strRuleContent<<"动作"<<iRulesNowNumber<<":制动至"<< fCarNowSpeed<<"km/h";
         }
         else if (fCarNowSpeed == fCarLastSpeed)
         {
             strRuleContent<<"动作"<<iRulesNowNumber<<":保持"<< fCarNowSpeed<<"km/h";
         }
        int iRet = DecisionDatabase::InsertLocationRuleInfoAndActionDirective(PlanRouteName.c_str(), iRulesNowNumber,
                         strRuleContent.str().c_str(), 0, iLocalNowNumber, msg->Longitude, msg->Latitude,
                         msg->Height, fCarNowSpeed);
        if (iRet < 0) {
             LOG(ERROR)<<Name()<<"StopPlanRoute insert iLocalNowNumber = "<< iLocalNowNumber<< "  last record error";
             iLocalNowNumber--;
             iRulesNowNumber--;
        }
     }
}

// 执行自动驾驶ＭＰＣ算法
void Decision::ExcuteSelfDriving(const localization::localMsg::ConstPtr& msg)
{
   // 判断当前RCU是否为自动驾驶模式
//    if (uiCarMode != CTR_MD_REQ_AUTO_DRIVE)
//    {
//        LOG(INFO)<<Name()<<" ExcuteSelfDriving uiCarMode is not auto drive.";
//        return;
//    }
    LOG(INFO)<<Name()<<" ilocalTotalNumbers::."<<ilocalTotalNumbers << "iLocalNowNumber:"<<iLocalNowNumber;
    if (CarAutoDriveStop())
        return;
    int iPredictPoints;
    if (ilocalTotalNumbers < (iLocalNowNumber + iPredictionPointsPerNumber)){
        iPredictPoints = ilocalTotalNumbers - iLocalNowNumber;
    }
    else{
        iPredictPoints = iPredictionPointsPerNumber;
    }
    if ((iLocalNowNumber > 0)&&((((msg->Longitude - ptsx[0])>0) && ((msg->Longitude - ptsx[1])< 0) ||
            ((msg->Longitude - ptsx[0])<0) && ((msg->Longitude - ptsx[1])> 0))&&
            (((msg->Latitude - ptsy[0])>0) && ((msg->Latitude - ptsy[1])< 0) ||
                        ((msg->Latitude - ptsy[0])<0) && ((msg->Latitude - ptsy[1])> 0))))
    {
        LOG(INFO)<<Name()<<" ExcuteSelfDriving plan route does not alter.";
    }
    else
    {
        std::vector<LOCATION_RULE_INFO> tempInfo;
        LOG(INFO)<<Name()<<" ExcuteSelfDriving QueryLocationRuleInfo start.";
        tempInfo = DecisionDatabase::QueryLocationRuleInfo(iLocalNowNumber, iPredictPoints);
        ptsx.clear();
        ptsy.clear();
        for (int i= 0; i< iPredictPoints; i++)
        {
           ptsx.push_back( tempInfo[i].LocationLon);
           ptsy.push_back( tempInfo[i].LocationLat);
        }
        mpc.ref_speed = tempInfo[1].LocationSpeed;
        LOG(INFO)<<Name()<<" ExcuteSelfDriving plan route  altered.";
    }
    double px = msg->Longitude;
    double py = msg->Latitude;
    double psi = deg2rad(msg->AngleDeviated + 90);
    double v =  (double)fCarNowSpeed;
    MPC_Controller(ptsx, ptsy, px, py, psi, v);
    UpdataCarRuleResults();
    iLocalNowNumber ++;
}

void Decision::TestMPC()
{
    if (CarAutoDriveStop())
        return;
    int iPredictPoints;
    if (ilocalTotalNumbers < (iLocalNowNumber + iPredictionPointsPerNumber)){
        iPredictPoints = ilocalTotalNumbers - iLocalNowNumber;
    }
    else{
        iPredictPoints = iPredictionPointsPerNumber;
    }
    if ((iLocalNowNumber > 0)&&((((vPx[iLocalNowNumber] - ptsx[0])>0) && ((vPx[iLocalNowNumber] - ptsx[1])< 0) ||
            ((vPx[iLocalNowNumber] - ptsx[0])<0) && ((vPx[iLocalNowNumber] - ptsx[1])> 0))&&
            (((vPy[iLocalNowNumber] - ptsy[0])>0) && ((vPy[iLocalNowNumber] - ptsy[1])< 0) ||
                        ((vPy[iLocalNowNumber] - ptsy[0])<0) && ((vPy[iLocalNowNumber]- ptsy[1])> 0))))
    {
        LOG(INFO)<<Name()<<" ExcuteSelfDriving plan route does not alter.";
    }
    else
    {
        std::vector<LOCATION_RULE_INFO> tempInfo;
        LOG(INFO)<<Name()<<" ExcuteSelfDriving QueryLocationRuleInfo start.";
        tempInfo = DecisionDatabase::QueryLocationRuleInfo(iLocalNowNumber, iPredictPoints);
        ptsx.clear();
        ptsy.clear();
        for (int i= 0; i< iPredictPoints; i++)
        {
           ptsx.push_back( tempInfo[i].LocationLon);
           ptsy.push_back( tempInfo[i].LocationLat);
        }
        mpc.ref_speed = tempInfo[1].LocationSpeed;
        LOG(INFO)<<Name()<<" ExcuteSelfDriving plan route  altered.";
    }

    double px = vPx[iLocalNowNumber];
    double py = vPy[iLocalNowNumber];
    double psi = deg2rad(40.45 + 90);
    double v =  (double)fCarNowSpeed;
    LOG(INFO)<<Name()<< " MPC start";
    MPC_Controller(ptsx, ptsy, px, py, psi, v);
    LOG(INFO)<<Name()<< " MPC end";
    //UpdataCarRuleResults();
    iLocalNowNumber ++;
}

// 自动驾驶停止
bool  Decision::CarAutoDriveStop()
{
    try{

        if (ilocalTotalNumbers == iLocalNowNumber){
            uiCarMode = CMD_AUTODRIE_END;
            double breakCtr = -5; // 踩刹车到50%
            ros::Rate loop_rate(1);
            while(ros::ok()){
                if (SpeedControl(breakCtr) == 1)
                {
                    if (BreakCtrControl(SFT_PSNG_LRN_REQ_PARK) == 1) // P挡
                        break;
                }
                ros::spinOnce();
                loop_rate.sleep();
            }
            decision::rulesMsg ruleMsg;
            if (std::abs(fCarNowSpeed - mpc.ref_speed) < 1) //低速（ <10km/h）：± 0.25km/h  高速（ >10km/h）：± 1km/h
            {
                 ruleMsg.DirectiveResult = RULE_EXCUTE_RESULT_SUCCESS;
            }
            else
            {
                ruleMsg.DirectiveResult = RULE_EXCUTE_RESULT_FIAL;
            }
            int iResult = ruleMsg.DirectiveResult;
            DecisionDatabase::UpdateActionDirective(PlanRouteName.c_str(), iRulesNowNumber, iResult);
            ruleMsg.DirectiveID = iRulesNowNumber;
            Rules_pub.publish(ruleMsg);
            return true;
        }
        else
            return false;
    }
    catch (ros::Exception ex)
    {
        LOG(ERROR)<<Name()<<":CarAutoDriveStop error."<< ex.what();
        return false;
    }
}


bool Decision::UpdataCarRuleResults()
{
    try{
        if (iLocalNowNumber >0 && ((iLocalNowNumber % iPredictionPointsPerNumber) == 0)){
            decision::rulesMsg ruleMsg;
            if (std::abs(fCarNowSpeed - mpc.ref_speed) < 1) //低速（ <10km/h）：± 0.25km/h  高速（ >10km/h）：± 1km/h
            {
                 ruleMsg.DirectiveResult = RULE_EXCUTE_RESULT_SUCCESS;
            }
            else
            {
                ruleMsg.DirectiveResult = RULE_EXCUTE_RESULT_FIAL;
            }
            int iResult = ruleMsg.DirectiveResult;
            DecisionDatabase::UpdateActionDirective(PlanRouteName.c_str(), iRulesNowNumber, iResult);
            ruleMsg.DirectiveID = iRulesNowNumber;
            Rules_pub.publish(ruleMsg);
            iRulesNowNumber ++;
        }
        return true;
    }
    catch (ros::Exception ex)
    {
        LOG(ERROR)<<Name()<<":UpdataCarRuleResults error."<< ex.what();
        return false;
    }
}


void Decision::MPC_Controller(std::vector<double> &ptsx, std::vector<double> &ptsy,
                              double &px, double &py,  double &psi, double &v)
{
     // 转换坐标到当前车辆坐标
     Eigen::MatrixXd waypoints = transformGlobal2Vehicle(px, py, psi, ptsx, ptsy);
     Eigen::VectorXd Ptsx = waypoints.row(0);
     Eigen::VectorXd Ptsy = waypoints.row(1);
     for (unsigned i=0; i < ptsx.size(); ++i) {
         LOG(INFO)<<Name()<< " lon: "<< Ptsx(i)<< " lat: "<<Ptsy(i);
     }
     // 获取状态值
     auto coeffs = polyfit(Ptsx, Ptsy, 3);
     double cte = evaluateCte(coeffs);
     double epsi = evaluateEpsi(coeffs);
     Eigen::VectorXd state(6);
     state << 0, 0, 0, v, cte, epsi;
     // 计算方向和油门，范围都在[-1, 1]之间
     // 这个地方有延迟，取第三个
     vector<vector<double>> result = mpc.Solve(state, coeffs);
     // 展示MPC预测的路径
     vector<double> mpc_x_vals=result[0];
     vector<double> mpc_y_vals=result[1];
     std::vector<double>::iterator iter_start = mpc_x_vals.begin();
     std::vector<double>::iterator iter_end = mpc_x_vals.end();
     std::vector<double>::iterator iter;
     int i = 0;
     for ( iter = iter_start; iter != iter_end; iter++)
     {
       LOG(INFO)<<Name()<<" MPC_Controller prediction LocationLon = "<<std::setprecision(16)<<mpc_x_vals[i]
             <<" LocationLat = "<< std::setprecision(16)<<mpc_y_vals[i];
       i++;
     }
     double steer_value = result[2][mpc.latency];
     double throttle_value = result[3][mpc.latency];
     mpc.delta_prev = steer_value;
     mpc.a_prev = throttle_value;
     LOG(INFO)<<Name()<< " stree alter value:"<<steer_value<<" throttle alter value:"<<throttle_value;
//     SpeedControl(steer_value);
//     StreeControl(throttle_value);
//     decision::controlMsg control;
//     control.nowSpeed = fCarNowSpeed;
//     control.refSpeed = mpc.ref_speed;
//     control.steer =  mpc.delta_prev;
//     control.throttle = mpc.a_prev;
//     control.cte = cte;
//     Control_pub.publish(control);
}

unsigned int Decision::SpeedControl(double &speed)
{

     canbus::PedalActCtrReq srv;
     if (mpc.a_prev >0){
         srv.request.uiAccPsngCtrReq =(unsigned int)(speed *10);
         srv.request.uiBrkPsngCtrReq =0;
     }
     else{
         srv.request.uiAccPsngCtrReq = 0;
         srv.request.uiBrkPsngCtrReq =(unsigned int)(speed *10);
     }
     srv.request.iAccSpdCtrReq = 20;
     srv.request.iBrkSpdCtrReq = 50;
     srv.request.uiBrkCtrMdReq = 1;
     srv.request.uiBrkFocCtrReq = 350;
     if (clientPedalActCtrReq.call(srv))
     {
         LOG(INFO)<<Name()<<" Speed Control service return success, value = "<<srv.response.uiReturn;
     }
     else
     {
         LOG(ERROR)<<Name()<<" Speed Control service return falil";
     }
    return srv.response.uiReturn;
}


unsigned int Decision::StreeControl(double &streer)
{

     canbus::StrActCtrReq srv;
     srv.request.iStrAngCtrReq =(unsigned int)(streer *100);
     srv.request.uiStrCtrMdReq =1;
     srv.request.iStrSpdCtrReq = 200;
     srv.request.iStrTrqCtrReq = 50;
     if (clientStrActCtrReq.call(srv))
     {
         LOG(INFO)<<Name()<<" stree Control service return success, value = "<<srv.response.uiReturn;
     }
     else
     {
         LOG(ERROR)<<Name()<<" stree Control service return falil";
     }
     return srv.response.uiReturn;
}


unsigned int Decision::BreakCtrControl(unsigned int breakmode)
{     
     canbus::SftActCtrReq srv;
     srv.request.uiSftPsngCtrReq = breakmode;
     srv.request.uiSftMdReq = SFT_MD_REQ_2_SECOND;
     srv.request.uiClchPsngCtrReq = 1;
     srv.request.iClchSpdCtrReq = 0;
     if (clientSftActCtrReq.call(srv))
     {
         LOG(INFO)<<Name()<<" break Control service return success, value = "<<srv.response.uiReturn;
     }
     else
     {
         LOG(ERROR)<<Name()<<" break Control service return falil";
     }
     return srv.response.uiReturn;
}

 // 根据输入值x,y解方程
 Eigen::VectorXd Decision::polyfit(Eigen::VectorXd xvals, Eigen::VectorXd yvals, int order) {
     assert(xvals.size() == yvals.size());
     assert(order >= 1 && order <= xvals.size() - 1);
     Eigen::MatrixXd A(xvals.size(), order + 1);
     for (int i = 0; i < xvals.size(); i++) {
         A(i, 0) = 1.0;
     }
     for (int j = 0; j < xvals.size(); j++) {
         for (int i = 0; i < order; i++) {
             A(j, i + 1) = A(j, i) * xvals(j);
         }
     }
     auto Q = A.householderQr();
     auto result = Q.solve(yvals);
     return result;
 }

// 计算CTE误差
 double Decision::evaluateCte(Eigen::VectorXd coeffs) {
   return polyeval(coeffs, 0);
 }

 // 计算Epsi误差
 double Decision::evaluateEpsi(Eigen::VectorXd coeffs) {
   return -atan(coeffs[1]);
 }

 // 把全局坐标转换为汽车坐标，以当前汽车的位置作为原点
 Eigen::MatrixXd Decision::transformGlobal2Vehicle(double x, double y, double psi,
        const vector<double> & ptsx, const vector<double> & ptsy) {
    assert(ptsx.size() == ptsy.size());
    unsigned len = ptsx.size();
    auto waypoints = Eigen::MatrixXd(2, len);
    for (auto i = 0; i < len; ++i) {
        waypoints(0, i) = cos(psi) * (ptsx[i] - x) + sin(psi) * (ptsy[i] - y);
        waypoints(1, i) = -sin(psi) * (ptsx[i] - x) + cos(psi) * (ptsy[i] - y);
    }
    return waypoints;
}

} // decisionspace
} // robot

