#include "perception.h"
#include "../common/message.h"
#include "../common/canmsgobdmsgid.h"
#include "../canbus/candatatype.h"
#include <perception/aesMsg.h>
#include <perception/slMsg.h>
#include <perception/seMsg.h>
#include <perception/si2Msg.h>
#include <perception/cecMsg.h>
#include <perception/eiMsg.h>
#include <perception/eeMsg.h>

namespace  robot {
namespace perception {


std::string Perception::Name() const { return "perception"; }

int Perception::Init()
{
    LOG(INFO) << Name() << " Init start...";

    node_handle_.reset(new ros::NodeHandle());

    LOG(INFO) << Name() << " Init end.";
    return ROBOT_SUCCESS;
}

int Perception::Start()
{
    LOG(INFO) << Name() << " Start start...";
    try
    {
        ros::Subscriber sub = node_handle_->subscribe(robot::common::CanDataAnalysis, 1000, &Perception::CanTopicCallback, this);

        bus_pub_6E5 = node_handle_->advertise<::perception::aesMsg>(robot::common::AesData, 2000);
        bus_pub_6EE = node_handle_->advertise<::perception::slMsg>(robot::common::SlData, 2000);
        bus_pub_6ED = node_handle_->advertise<::perception::si2Msg>(robot::common::Si2Data, 2000);

        bus_pub_6D1 = node_handle_->advertise<::perception::seMsg>(robot::common::SeData, 2000);

        bus_pub_6D2 = node_handle_->advertise<::perception::cecMsg>(robot::common::AcceData, 2000);
        bus_pub_6D3 = node_handle_->advertise<::perception::cecMsg>(robot::common::BrkeData, 2000);
        bus_pub_6D4 = node_handle_->advertise<::perception::cecMsg>(robot::common::XsfteData, 2000);
        bus_pub_6D5 = node_handle_->advertise<::perception::cecMsg>(robot::common::YsfteData, 2000);
        bus_pub_6D6 = node_handle_->advertise<::perception::cecMsg>(robot::common::StreData, 2000);

        bus_pub_2E0 = node_handle_->advertise<::perception::eiMsg>(robot::common::EiData, 2000);
        bus_pub_2E1 = node_handle_->advertise<::perception::eeMsg>(robot::common::EeData, 2000);

        ros::spin();
    }
    catch(ros::Exception e)
    {
         LOG(INFO) << Name() << " Start exception is:" << e.what();
         return ROBOT_FAILTURE;
    }  
    LOG(INFO) << Name() << " Start end.";
    return ROBOT_SUCCESS;
}

void Perception::Stop()
{
   LOG(INFO) << Name() << " Stop start...";
   LOG(INFO) << Name() << " Stop end.";
}


void Perception::CanTopicCallback(const canbus::canMsg::ConstPtr& msg)
{
    LOG(INFO) << " Msg CanData :" << "ID:" << msg->id << ",Data:" << msg->data;

    switch (msg->id) {

    case robot::common::ActuatorEnableStatus:
    {
        PublishState_6E5(msg);
    }
        break;
    case robot::common::RSysInfoBack2:
    {
        PublishState_6ED(msg);
    }
        break;
    case robot::common::SysLearnBack:
    {
        PublishState_6EE(msg);
    }
        break;
    case robot::common::SysWarnErr:
    {
        PublishState_6D1(msg);
    }
        break;
    case robot::common::AccErr:
    {
        PublishState_6D2_6(msg, 2, bus_pub_6D2);
    }
        break;
    case robot::common::BrkErr:
    {
        PublishState_6D2_6(msg, 3, bus_pub_6D3);
    }
        break;
    case robot::common::XsftErr:
    {
        PublishState_6D2_6(msg, 4, bus_pub_6D4);
    }
        break;
    case robot::common::YsftErr:
    {
        PublishState_6D2_6(msg, 5, bus_pub_6D5);
    }
        break;
    case robot::common::StrErr:
    {
        PublishState_6D2_6(msg, 6, bus_pub_6D6);
    }
        break;
    case robot::common::EstopInfo:
    {
        PublishState_2E0(msg);
    }
        break;
    case robot::common::EstopErr:
    {
        PublishState_2E1(msg);
    }
        break;
    default:
        break;
    }
}

bool Perception::PublishState_6E5(const canbus::canMsg::ConstPtr &msg)
{
    static robot::canbusSpace::T_STATE_OUT_6E5 s_state_6E5;

    int copySize;
    if(msg->data.size() < sizeof(s_state_6E5))
        copySize = msg->data.size();
    else
        copySize = sizeof(s_state_6E5);

    robot::canbusSpace::T_STATE_OUT_6E5 state_6E5;
    memset(&state_6E5, 0 ,sizeof(state_6E5));
    memcpy(&state_6E5, msg->data.c_str(), copySize);

    if(state_6E5.uiAccClchCtrStat != s_state_6E5.uiAccClchCtrStat || state_6E5.uiAccMotEnStat != s_state_6E5.uiAccMotEnStat ||
            state_6E5.uiBrkHVFFStat != s_state_6E5.uiBrkHVFFStat || state_6E5.uiBrkMotEnStat != s_state_6E5.uiBrkMotEnStat ||
            state_6E5.uiClchMotEnStat != s_state_6E5.uiClchMotEnStat || state_6E5.uiPauseEnStat != s_state_6E5.uiPauseEnStat ||
            state_6E5.uiStrMotEnStat != s_state_6E5.uiStrMotEnStat || state_6E5.uiXSftMotEnStat != s_state_6E5.uiXSftMotEnStat ||
            state_6E5.uiYSftMotEnStat != s_state_6E5.uiYSftMotEnStat)
    {

        memcpy(&s_state_6E5, msg->data.c_str(), copySize);
        if(msg->data.size() < sizeof(s_state_6E5))
            memset((char*)&s_state_6E5 + msg->data.size(), 0, sizeof(s_state_6E5) - copySize);
        if(bus_pub_6E5)
        {
            ::perception::aesMsg msg;
            msg.AccClchCtrStat = state_6E5.uiAccClchCtrStat;
            msg.AccMotEnStat =  state_6E5.uiAccMotEnStat;
            msg.BrkHVFFStat = state_6E5.uiBrkHVFFStat;
            msg.BrkMotEnStat = state_6E5.uiBrkMotEnStat;
            msg.ClchMotEnStat = state_6E5.uiClchMotEnStat;
            msg.PauseEnStat = state_6E5.uiPauseEnStat;
            msg.StrMotEnStat = state_6E5.uiStrMotEnStat;
            msg.XSftMotEnStat = state_6E5.uiXSftMotEnStat;
            msg.YSftMotEnStat = state_6E5.uiYSftMotEnStat;
            bus_pub_6E5.publish(msg);
        }
        else
        {
            LOG(INFO) << Name() << " PublishState_6E5 publisher is unvalid";
            return false;
        }
        return true;
    }
    else
    {
        LOG(INFO) << Name() << " PublishState_6E5 data is repeat";
        return false;
    }
}

bool Perception::PublishState_6EE(const canbus::canMsg::ConstPtr &msg)
{
    static robot::canbusSpace::T_STATE_OUT_6EE s_state_6EE;

    int copySize;
    if(msg->data.size() < sizeof(s_state_6EE))
        copySize = msg->data.size();
    else
        copySize = sizeof(s_state_6EE);

    robot::canbusSpace::T_STATE_OUT_6EE state_6EE;
    memset(&state_6EE, 0 ,sizeof(state_6EE));
    memcpy(&state_6EE, msg->data.c_str(), copySize);

    if(state_6EE.uiAcc0LrnStat != s_state_6EE.uiAcc0LrnStat || state_6EE.uiAccLrnStat != s_state_6EE.uiAccLrnStat ||
            state_6EE.uiAccMaxLrnStat != s_state_6EE.uiAccMaxLrnStat || state_6EE.uiBrk0LrnStat != s_state_6EE.uiBrk0LrnStat ||
            state_6EE.uiBrkLrnStat != s_state_6EE.uiBrkLrnStat || state_6EE.uiBrkMaxLrnStat != s_state_6EE.uiBrkMaxLrnStat ||
            state_6EE.uiClch0LrnStat != s_state_6EE.uiClch0LrnStat || state_6EE.uiClchLrnStat != s_state_6EE.uiClchLrnStat ||
            state_6EE.uiClchMaxLrnStat != s_state_6EE.uiClchMaxLrnStat || state_6EE.uiDLrnStat != s_state_6EE.uiDLrnStat ||
            state_6EE.uiDpLrnStat != s_state_6EE.uiDpLrnStat || state_6EE.uiMLrnStat != s_state_6EE.uiMLrnStat ||
            state_6EE.uiNLrnStat != s_state_6EE.uiNLrnStat || state_6EE.uiPLrnStat != s_state_6EE.uiPLrnStat ||
            state_6EE.uiRLrnStat != s_state_6EE.uiRLrnStat || state_6EE.uiSftLrnStat != s_state_6EE.uiSftLrnStat ||
            state_6EE.uiStr0LrnStat != s_state_6EE.uiStr0LrnStat || state_6EE.uiStrLLrnStat != s_state_6EE.uiStrLLrnStat ||
            state_6EE.uiStrLrnStat != s_state_6EE.uiStrLrnStat || state_6EE.uiStrRLrnStat != s_state_6EE.uiStrRLrnStat ||
            state_6EE.uiUpLrnStat != s_state_6EE.uiUpLrnStat)
    {

        memcpy(&s_state_6EE, msg->data.c_str(), copySize);
        if(msg->data.size() < sizeof(s_state_6EE))
            memset((char*)&s_state_6EE + msg->data.size(), 0, sizeof(s_state_6EE) - copySize);
        if(bus_pub_6EE)
        {
            ::perception::slMsg msg;
            msg.Acc0LrnStat = state_6EE.uiAcc0LrnStat;
            msg.AccLrnStat = state_6EE.uiAccLrnStat;
            msg.AccMaxLrnStat = state_6EE.uiAccMaxLrnStat;
            msg.Brk0LrnStat = state_6EE.uiBrk0LrnStat;
            msg.BrkLrnStat = state_6EE.uiBrkLrnStat;
            msg.BrkMaxLrnStat = state_6EE.uiBrkMaxLrnStat;
            msg.Clch0LrnStat = state_6EE.uiClch0LrnStat;
            msg.ClchLrnStat = state_6EE.uiClchLrnStat;
            msg.ClchMaxLrnStat = state_6EE.uiClchMaxLrnStat;
            msg.DLrnStat = state_6EE.uiDLrnStat;
            msg.DpLrnStat = state_6EE.uiDpLrnStat;
            msg.MLrnStat = state_6EE.uiMLrnStat;
            msg.NLrnStat = state_6EE.uiNLrnStat;
            msg.PLrnStat = state_6EE.uiPLrnStat;
            msg.RLrnStat = state_6EE.uiRLrnStat;
            msg.SftLrnStat = state_6EE.uiSftLrnStat;
            msg.Str0LrnStat = state_6EE.uiStr0LrnStat;
            msg.StrLLrnStat = state_6EE.uiStrLLrnStat;
            msg.StrLrnStat = state_6EE.uiStrLrnStat;
            msg.StrRLrnStat = state_6EE.uiStrRLrnStat;
            msg.UpLrnStat = state_6EE.uiPLrnStat;
            bus_pub_6EE.publish(msg);
        }
        else
        {
            LOG(INFO) << Name() << " PublishState_6EE publisher is unvalid";
            return false;
        }
        return true;
    }
    else
    {
        LOG(INFO) << Name() << " PublishState_6EE data is repeat";
        return false;
    }
}

bool Perception::PublishState_6ED(const canbus::canMsg::ConstPtr &msg)
{
    static robot::canbusSpace::T_STATE_OUT_6ED s_state_6ED;

    int copySize;
    if(msg->data.size() < sizeof(s_state_6ED))
        copySize = msg->data.size();
    else
        copySize = sizeof(s_state_6ED);

    robot::canbusSpace::T_STATE_OUT_6ED state_6ED;
    memset(&state_6ED, 0 ,sizeof(state_6ED));
    memcpy(&state_6ED, msg->data.c_str(), copySize);

    if(state_6ED.uiAccCtrAlive != s_state_6ED.uiAccCtrAlive || state_6ED.uiAccMotClbr != s_state_6ED.uiAccMotClbr ||
            state_6ED.uiBrkCtrAlive != s_state_6ED.uiBrkCtrAlive || state_6ED.uiBrkMotClbr != s_state_6ED.uiBrkMotClbr ||
            state_6ED.uiClchCtrAlive != s_state_6ED.uiClchCtrAlive || state_6ED.uiClchMotClbr != s_state_6ED.uiClchMotClbr ||
            state_6ED.uiCtrMdStat != s_state_6ED.uiCtrMdStat || state_6ED.uiRCUAlive != s_state_6ED.uiRCUAlive ||
            state_6ED.uiStrCtrAlive != s_state_6ED.uiStrCtrAlive || state_6ED.uiStrMotClbr != s_state_6ED.uiStrMotClbr ||
            state_6ED.uiXSftArmCtrAlive != s_state_6ED.uiXSftArmCtrAlive || state_6ED.uiXSftMotClbr != s_state_6ED.uiXSftMotClbr ||
            state_6ED.uiYSftArmCtrAlive != s_state_6ED.uiYSftArmCtrAlive || state_6ED.uiYSftMotClbr != s_state_6ED.uiYSftMotClbr)
    {

        memcpy(&s_state_6ED, msg->data.c_str(), copySize);
        if(msg->data.size() < sizeof(s_state_6ED))
            memset((char*)&s_state_6ED + msg->data.size(), 0, sizeof(s_state_6ED) - copySize);
        if(bus_pub_6ED)
        {
            ::perception::si2Msg msg;
            msg.AccCtrAlive = state_6ED.uiAccCtrAlive;
            msg.AccMotClbr = state_6ED.uiAccMotClbr;
            msg.BrkCtrAlive = state_6ED.uiBrkCtrAlive;
            msg.BrkMotClbr = state_6ED.uiBrkMotClbr;
            msg.ClchCtrAlive = state_6ED.uiClchCtrAlive;
            msg.ClchMotClbr = state_6ED.uiClchMotClbr;
            msg.CtrMdStat = state_6ED.uiCtrMdStat;
            msg.RCUAlive = state_6ED.uiRCUAlive;
            msg.StrCtrAlive = state_6ED.uiStrCtrAlive;
            msg.StrMotClbr = state_6ED.uiStrMotClbr;
            msg.XSftArmCtrAlive = state_6ED.uiXSftArmCtrAlive;
            msg.XSftMotClbr = state_6ED.uiXSftMotClbr;
            msg.YSftArmCtrAlive = state_6ED.uiYSftArmCtrAlive;
            msg.YSftMotClbr = state_6ED.uiYSftMotClbr;
            bus_pub_6ED.publish(msg);
        }
        else
        {
            LOG(INFO) << Name() << " PublishState_6ED publisher is unvalid";
            return false;
        }
        return true;
    }
    else
    {
        LOG(INFO) << Name() << " PublishState_6ED data is repeat";
        return false;
    }
}

bool Perception::PublishState_6D1(const canbus::canMsg::ConstPtr &msg)
{
    static robot::canbusSpace::T_STATE_OUT_6D1 s_state_6D1;

    int copySize;
    if(msg->data.size() < sizeof(s_state_6D1))
        copySize = msg->data.size();
    else
        copySize = sizeof(s_state_6D1);

    robot::canbusSpace::T_STATE_OUT_6D1 state_6D1;
    memset(&state_6D1, 0 ,sizeof(state_6D1));
    memcpy(&state_6D1, msg->data.c_str(), copySize);

    if(state_6D1.uiSysErra != s_state_6D1.uiSysErra || state_6D1.uiSysErrb != s_state_6D1.uiSysErrb || state_6D1.uiSysErrc != s_state_6D1.uiSysErrc ||
            state_6D1.uiSysErrd != s_state_6D1.uiSysErrd || state_6D1.uiSysErre != s_state_6D1.uiSysErre || state_6D1.uiSysErrf != s_state_6D1.uiSysErrf ||
            state_6D1.uiSysErrg != s_state_6D1.uiSysErrg || state_6D1.uiSysErrh != s_state_6D1.uiSysErrh || state_6D1.uiSysErri != s_state_6D1.uiSysErri ||
            state_6D1.uiSysErrj != s_state_6D1.uiSysErrj || state_6D1.uiSysErrk != s_state_6D1.uiSysErrk || state_6D1.uiSysErrl != s_state_6D1.uiSysErrl ||
            state_6D1.uiSysErrm != s_state_6D1.uiSysErrm || state_6D1.uiSysErrn != s_state_6D1.uiSysErrn || state_6D1.uiSysErro != s_state_6D1.uiSysErro ||
            state_6D1.uiSysErrp != s_state_6D1.uiSysErrp || state_6D1.uiSysErrq != s_state_6D1.uiSysErrq || state_6D1.uiSysErrr != s_state_6D1.uiSysErrr ||
            state_6D1.uiSysErrs != s_state_6D1.uiSysErrs || state_6D1.uiSysErrt != s_state_6D1.uiSysErrt || state_6D1.uiSysErru != s_state_6D1.uiSysErru ||
            state_6D1.uiSysErrv != s_state_6D1.uiSysErrv || state_6D1.uiSysErrw != s_state_6D1.uiSysErrw || state_6D1.uiSysErrx != s_state_6D1.uiSysErrx ||
            state_6D1.uiSysErry != s_state_6D1.uiSysErry || state_6D1.uiSysErrz != s_state_6D1.uiSysErrz)
    {

        memcpy(&s_state_6D1, msg->data.c_str(), copySize);
        if(msg->data.size() < sizeof(s_state_6D1))
            memset((char*)&s_state_6D1 + msg->data.size(), 0, sizeof(s_state_6D1) - copySize);
        if(bus_pub_6D1)
        {
            ::perception::seMsg msg;
            msg.SysErra = state_6D1.uiSysErra;
            msg.SysErrb = state_6D1.uiSysErrb;
            msg.SysErrc = state_6D1.uiSysErrc;
            msg.SysErrd = state_6D1.uiSysErrd;
            msg.SysErre = state_6D1.uiSysErre;
            msg.SysErrf = state_6D1.uiSysErrf;
            msg.SysErrg = state_6D1.uiSysErrg;
            msg.SysErrh = state_6D1.uiSysErrh;
            msg.SysErri = state_6D1.uiSysErri;
            msg.SysErrj = state_6D1.uiSysErrj;
            msg.SysErrk = state_6D1.uiSysErrk;
            msg.SysErrl = state_6D1.uiSysErrl;
            msg.SysErrm = state_6D1.uiSysErrm;
            msg.SysErrn = state_6D1.uiSysErrn;
            msg.SysErro = state_6D1.uiSysErro;
            msg.SysErrp = state_6D1.uiSysErrp;
            msg.SysErrq = state_6D1.uiSysErrq;
            msg.SysErrr = state_6D1.uiSysErrr;
            msg.SysErrs = state_6D1.uiSysErrs;
            msg.SysErrt = state_6D1.uiSysErrt;
            msg.SysErru = state_6D1.uiSysErru;
            msg.SysErrv = state_6D1.uiSysErrv;
            msg.SysErrw = state_6D1.uiSysErrw;
            msg.SysErrx = state_6D1.uiSysErrx;
            msg.SysErry = state_6D1.uiSysErry;
            msg.SysErrz = state_6D1.uiSysErrz;
            bus_pub_6D1.publish(msg);
        }
        else
        {
            LOG(INFO) << Name() << " PublishState_6D1 publisher is unvalid";
            return false;
        }
        return true;
    }
    else
    {
        LOG(INFO) << Name() << " PublishState_6D1 data is repeat";
        return false;
    }
}

bool Perception::PublishState_6D2_6(const canbus::canMsg::ConstPtr &msg, int type, const ros::Publisher &pub)
{
    static robot::canbusSpace::T_ACC_ERR s_state_6D2;
    static robot::canbusSpace::T_BRK_ERR s_state_6D3;
    static robot::canbusSpace::T_XSFT_ERR s_state_6D4;
    static robot::canbusSpace::T_YSFT_ERR s_state_6D5;
    static robot::canbusSpace::T_STR_ERR s_state_6D6;

    robot::canbusSpace::T_ACC_ERR *pErr;
    switch (type) {
    case 2:
        pErr = &s_state_6D2;
        break;
    case 3:
        pErr = (robot::canbusSpace::T_ACC_ERR *)&s_state_6D3;
        break;
    case 4:
        pErr = (robot::canbusSpace::T_ACC_ERR *)&s_state_6D4;
        break;
    case 5:
        pErr = (robot::canbusSpace::T_ACC_ERR *)&s_state_6D5;
        break;
    case 6:
        pErr = (robot::canbusSpace::T_ACC_ERR *)&s_state_6D6;
        break;
    default:
        LOG(INFO) << Name() << " PublishState_6D2_6 " << type << " type is unvalid";
        return false;
        break;
    }

    int copySize;
    if(msg->data.size() < sizeof(s_state_6D2))
        copySize = msg->data.size();
    else
        copySize = sizeof(s_state_6D2);

    //6D2_2 6D2_3 6D2_4 6D2_5 6D2_6 数据结构相同
    robot::canbusSpace::T_ACC_ERR state_6D2;
    memset(&state_6D2, 0 ,sizeof(state_6D2));
    memcpy(&state_6D2, msg->data.c_str(), copySize);

    if(state_6D2.uiAccErra != pErr->uiAccErra || state_6D2.uiAccErraa != pErr->uiAccErraa || state_6D2.uiAccErrab != pErr->uiAccErrab ||
            state_6D2.uiAccErrac != pErr->uiAccErrac || state_6D2.uiAccErrad != pErr->uiAccErrad || state_6D2.uiAccErrae != pErr->uiAccErrae ||
            state_6D2.uiAccErraf != pErr->uiAccErraf || state_6D2.uiAccErrag != pErr->uiAccErrag || state_6D2.uiAccErrah != pErr->uiAccErrah ||
            state_6D2.uiAccErrai != pErr->uiAccErrai || state_6D2.uiAccErrb != pErr->uiAccErrb || state_6D2.uiAccErrc != pErr->uiAccErrc ||
            state_6D2.uiAccErrd != pErr->uiAccErrd || state_6D2.uiAccErre != pErr->uiAccErre || state_6D2.uiAccErrf != pErr->uiAccErrf ||
            state_6D2.uiAccErrg != pErr->uiAccErrg || state_6D2.uiAccErrh != pErr->uiAccErrh || state_6D2.uiAccErrai != pErr->uiAccErrai ||
            state_6D2.uiAccErrj != pErr->uiAccErrj || state_6D2.uiAccErrk != pErr->uiAccErrk || state_6D2.uiAccErrl != pErr->uiAccErrl ||
            state_6D2.uiAccErrm != pErr->uiAccErrm || state_6D2.uiAccErrn != pErr->uiAccErrn || state_6D2.uiAccErro != pErr->uiAccErro ||
            state_6D2.uiAccErrp != pErr->uiAccErrp || state_6D2.uiAccErrq != pErr->uiAccErrq || state_6D2.uiAccErrr != pErr->uiAccErrr ||
            state_6D2.uiAccErrs != pErr->uiAccErrs || state_6D2.uiAccErrt != pErr->uiAccErrt || state_6D2.uiAccErru != pErr->uiAccErru ||
            state_6D2.uiAccErrv != pErr->uiAccErrv || state_6D2.uiAccErrw != pErr->uiAccErrw || state_6D2.uiAccErrx != pErr->uiAccErrx ||
            state_6D2.uiAccErry != pErr->uiAccErry || state_6D2.uiAccErrz != pErr->uiAccErrz)
    {

        memcpy(pErr, msg->data.c_str(), copySize);
        if(msg->data.size() < sizeof(s_state_6D2))
            memset((char*)&s_state_6D2 + msg->data.size(), 0, sizeof(s_state_6D2) - copySize);
        if(pub)
        {
            ::perception::cecMsg msg;
            msg.a = state_6D2.uiAccErra;
            msg.aa = state_6D2.uiAccErraa;
            msg.ab = state_6D2.uiAccErrab;
            msg.ac = state_6D2.uiAccErrac;
            msg.ad = state_6D2.uiAccErrad;
            msg.ae = state_6D2.uiAccErrae;
            msg.af = state_6D2.uiAccErraf;
            msg.ag = state_6D2.uiAccErrag;
            msg.ah = state_6D2.uiAccErrah;
            msg.ai = state_6D2.uiAccErrai;
            msg.b = state_6D2.uiAccErrb;
            msg.c = state_6D2.uiAccErrc;
            msg.d = state_6D2.uiAccErrd;
            msg.e = state_6D2.uiAccErre;
            msg.f = state_6D2.uiAccErrf;
            msg.g = state_6D2.uiAccErrg;
            msg.h = state_6D2.uiAccErrh;
            msg.i = state_6D2.uiAccErri;
            msg.j = state_6D2.uiAccErrj;
            msg.k = state_6D2.uiAccErrk;
            msg.l = state_6D2.uiAccErrl;
            msg.m = state_6D2.uiAccErrm;
            msg.n = state_6D2.uiAccErrn;
            msg.o = state_6D2.uiAccErro;
            msg.p = state_6D2.uiAccErrp;
            msg.q = state_6D2.uiAccErrq;
            msg.r = state_6D2.uiAccErrr;
            msg.s = state_6D2.uiAccErrs;
            msg.t = state_6D2.uiAccErrt;
            msg.u = state_6D2.uiAccErru;
            msg.v = state_6D2.uiAccErrv;
            msg.w = state_6D2.uiAccErrw;
            msg.x = state_6D2.uiAccErrx;
            msg.y = state_6D2.uiAccErry;
            msg.z = state_6D2.uiAccErrz;
            pub.publish(msg);
        }
        else
        {
            LOG(INFO) << Name() << " PublishState_6D2_6 publisher " << type <<" is unvalid";
            return false;
        }
        return true;
    }
    else
    {
        LOG(INFO) << Name() << " PublishState_6D2_6 " << type << " data is repeat";
        return false;
    }
}

bool Perception::PublishState_2E0(const canbus::canMsg::ConstPtr &msg)
{
    static robot::canbusSpace::T_ESTOP_2E0 s_state_2E0;

    int copySize;
    if(msg->data.size() < sizeof(s_state_2E0))
    {
        copySize = msg->data.size();
    }
    else
        copySize = sizeof(s_state_2E0);

    robot::canbusSpace::T_ESTOP_2E0 state_2E0;
    memset(&state_2E0, 0 ,sizeof(state_2E0));
    memcpy(&state_2E0, msg->data.c_str(), copySize);

    if(state_2E0.uiEstpAbnStat != s_state_2E0.uiEstpAbnStat || state_2E0.uiEstpAlive != state_2E0.uiEstpAlive ||
            state_2E0.uiEstpEnStat != s_state_2E0.uiEstpEnStat || state_2E0.uiPmpEnStat != s_state_2E0.uiPmpEnStat )
    {

        memcpy(&s_state_2E0, msg->data.c_str(), copySize);
        if(msg->data.size() < sizeof(s_state_2E0))
            memset((char*)&s_state_2E0 + msg->data.size(), 0, sizeof(s_state_2E0) - copySize);
        if(bus_pub_2E0)
        {
            ::perception::eiMsg msg;
            msg.EstpAbnStat = state_2E0.uiEstpAbnStat;
            msg.EstpAlive = state_2E0.uiEstpAlive;
            msg.EstpEnStat = state_2E0.uiEstpEnStat;
            msg.PmpEnStat = state_2E0.uiPmpEnStat;
            bus_pub_2E0.publish(msg);
        }
        else
        {
            LOG(INFO) << Name() << " PublishState_2E0 publisher is unvalid";
            return false;
        }
        return true;
    }
    else
    {
        LOG(INFO) << Name() << " PublishState_2E0 data is repeat";
        return false;
    }
}

bool Perception::PublishState_2E1(const canbus::canMsg::ConstPtr &msg)
{
    static robot::canbusSpace::T_ESTOP_2E1 s_state_2E1;

    int copySize;
    if(msg->data.size() < sizeof(s_state_2E1))
        copySize = msg->data.size();
    else
        copySize = sizeof(s_state_2E1);

    robot::canbusSpace::T_ESTOP_2E1 state_2E1;
    memset(&state_2E1, 0 ,sizeof(state_2E1));
    memcpy(&state_2E1, msg->data.c_str(), copySize);

    if(state_2E1.uiEspErra != s_state_2E1.uiEspErra || state_2E1.uiEspErrb != s_state_2E1.uiEspErrb || state_2E1.uiEspErrc != s_state_2E1.uiEspErrc ||
            state_2E1.uiEspErrd != s_state_2E1.uiEspErrd || state_2E1.uiEspErre != s_state_2E1.uiEspErre || state_2E1.uiEspErrf != s_state_2E1.uiEspErrf ||
            state_2E1.uiEspErrg != s_state_2E1.uiEspErrg || state_2E1.uiEspErrh != s_state_2E1.uiEspErrh || state_2E1.uiEspErri != s_state_2E1.uiEspErri ||
            state_2E1.uiEspErrj != s_state_2E1.uiEspErrj)
    {

        memcpy(&s_state_2E1, msg->data.c_str(), copySize);
        if(msg->data.size() < sizeof(s_state_2E1))
            memset((char*)&s_state_2E1 + msg->data.size(), 0, sizeof(s_state_2E1) - copySize);
        if(bus_pub_2E1)
        {
            ::perception::eeMsg msg;
            msg.uiEspErra = state_2E1.uiEspErra;
            msg.uiEspErrb = state_2E1.uiEspErrb;
            msg.uiEspErrc = state_2E1.uiEspErrc;
            msg.uiEspErrd = state_2E1.uiEspErrd;
            msg.uiEspErre = state_2E1.uiEspErre;
            msg.uiEspErrf = state_2E1.uiEspErrf;
            msg.uiEspErrg = state_2E1.uiEspErrg;
            msg.uiEspErrh = state_2E1.uiEspErrh;
            msg.uiEspErri = state_2E1.uiEspErri;
            msg.uiEspErrj = state_2E1.uiEspErrj;
            bus_pub_2E1.publish(msg);
        }
        else
        {
            LOG(INFO) << Name() << " PublishState_2E1 publisher is unvalid";
            return false;
        }
        return true;
    }
    else
    {
        LOG(INFO) << Name() << " PublishState_2E1 data is repeat";
        return false;
    }
}
    }  // namespace perception
}  // namespace robot
