#ifndef ROBOT_ROBOT_PERCEPTION_H
#define ROBOT_ROBOT_PERCEPTION_H

#include "../common/robotapp.h"
#include "ros/ros.h"
#include "std_msgs/String.h"
#include <canbus/canMsg.h>

namespace robot {
namespace perception {

#define ROBOT_SUCCESS     0
#define ROBOT_FAILTURE   -1

class Perception : public robot::common::RobotApp
{
public:
    Perception() {}

    /**
    * perception 节点名称
    */
    std::string Name() const override;

    /**
    * @brief module initialization function
    * @return initialization status
    */
    int Init() override;

    /**
    * @brief module start function
    * @return start status
    */
    int Start() override;

    /**
    * @brief module stop function
    */
    void Stop() override;

    /**
    * @brief Can topic data callback function
    */

    void CanTopicCallback(const canbus::canMsg::ConstPtr& msg);

    /**
    * @brief Publish State_6E5 topic
    */

    bool PublishState_6E5(const canbus::canMsg::ConstPtr& msg);

    /**
    * @brief Publish State_6ED topic
    */

    bool PublishState_6ED(const canbus::canMsg::ConstPtr& msg);

    /**
    * @brief Publish State_6EE topic
    */

    bool PublishState_6EE(const canbus::canMsg::ConstPtr& msg);

    /**
    * @brief Publish State_6D1 topic
    */

    bool PublishState_6D1(const canbus::canMsg::ConstPtr& msg);

    /**
    * @brief Publish State_6D2~State_6D6 topic
    */

    bool PublishState_6D2_6(const canbus::canMsg::ConstPtr& msg, int type, const ros::Publisher &pub);

    /**
    * @brief Publish State_2E0 topic
    */

    bool PublishState_2E0(const canbus::canMsg::ConstPtr& msg);

    /**
    * @brief Publish State_2E1 topic
    */

    bool PublishState_2E1(const canbus::canMsg::ConstPtr& msg);

private:

     ros::Publisher bus_pub_6E5;
     ros::Publisher bus_pub_6ED;
     ros::Publisher bus_pub_6EE;

     ros::Publisher bus_pub_6D1;

     ros::Publisher bus_pub_6D2;  
     ros::Publisher bus_pub_6D3;
     ros::Publisher bus_pub_6D4;
     ros::Publisher bus_pub_6D5;
     ros::Publisher bus_pub_6D6;

     ros::Publisher bus_pub_2E0;
     ros::Publisher bus_pub_2E1;
};


}  // namespace perception
}  // namespace robot

#endif // ROBOT_ROBOT_PERCEPTION_H
